#include "avd3371.h"
#include <utility.h>
#include <userint.h>

/*****************************************************************************/
/*   Copyright 1998 National Instruments Corporation.  All Rights Reserved.  */
/*****************************************************************************/


/*====================== D3371 Transmission Analyzer ========================*/
/* LabWindows/CVI 5.0 Instrument Driver										 */
/* Original Release: June/01/2001											 */
/* By: Hitoshi Takahashi													 */
/*					 Supervisor: Yohei Hirakoso								 */
/*                   SPSpj Gyoda R&D,  ADVANTEST CORPORATION				 */
/*                   Home Page;:http://www.advantest.co.jp					 */
/*                   e-mail;:   info@advantest.co.jp						 */
/*---------------------------------------------------------------------------*/
/* Modification History: 													 */
/*         Rev 1.0 :  June/01/2001											 */
/*																			 */
/*===========================================================================*/
 

#include <visa.h>
#include <ansi_c.h>
#include <formatio.h>
#include "avd3371.h"

#define AVD3371_REVISION   "Rev 1.0, 07/2001, CVI 5.0" /*  Instrument driver revision */
#define BUFFER_SIZE         1024L 					   /* File I/O buffer size */
    

/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT COMMAND ARRAYS =====================================*/
/*****************************************************************************/

/* Signal Source*/
static ViString sigSrcArr[] = {"EXT", "INT"};

/* Polarization */
static ViString polArr[] = {"INV", "NORM"};

/* Mark Rate */
static ViString mkrRtoArr[] = {"MRAT0", "MRAT1", "MRAT2", "MRAT4",
							   "MRAT6", "MRAT7", "MRAT8", "MINV4"};

/* Add Error Mode */
static ViString addErrModeArr[] = {"SING", "REP", "EXT"};
						
/* Data, Clock Output */
static ViString offsetModeArr[] = {"HIGH", "MIDD", "LOW"};
static ViString termVoltOutArr[] = {"P1R3V", "GND", "M2V"};

/* Trigger Source */
static ViString trgSrcArr[] = {"CLOC8", "CLOC32", "PATT"};
static ViString trgOutSrcArr[] = {"CLOC16", "PATT", "SYNC"};

/* Auto Synchronization*/
static ViString autoSyncArr[] = {"0", "1", "ONCE"};

/* Data, Clock Input */
static ViString termVoltInpArr[] = {"P3V", "P1R3V", "GND", "M2V"};

/* Set Measurement Condition */
static ViString timerModeArr[] = {"SING", "REP", "UNT"};
static ViString msIntrvlArr[]= {"0.1", "1.0"};
static ViString errRateThresArr[] = {"1E-3,1E-6", "1E-4,1E-8"};

/* Display Format */
static ViString frmtIntExpArr[] = {"INT", "EXP"};
static ViString frmtIntPrcArr[] = {"INT", "PERC"};

/* Load */
static ViString loadArr[] = {"MMEM:LOAD:SET", "MMEM:LOAD:SSG:SET", "MMEM:LOAD:PPG:SET",
							 "MMEM:LOAD:PPG:PROG", "MMEM:LOAD:ED:SET", "MMEM:LOAD:ED:PROG"};

/* Save */
static ViString saveArr[] = {"MMEM:STOR:SET", "MMEM:STOR:SSG:SET", "MMEM:STOR:PPG:SET",
							 "MMEM:STOR:PPG:PROG", "MMEM:STOR:ED:SET", "MMEM:STOR:ED:PROG",
							 "MMEM:STOR:ED:BME:LOG"};

/* GPIB */
static ViString directArr[] = {"NTR", "PTR"};

/* Threshold Condition */
static ViString thresCndArr[] = {"EM3", "EM4", "EM5", "EM6", "EM7", "EM8", "LEM8"};

/*****************************************************************************/
/*= Display Format for FETCh/CFETch =========================================*/
/*****************************************************************************/

#define DISP_DBL  VI_ON
#define DISP_INT  VI_OFF

ViBoolean dispBitCnt = DISP_DBL;
ViBoolean dispErrCnt = DISP_DBL;
ViBoolean dispImmErrCnt = DISP_DBL;
ViBoolean dispErrIntrvl = DISP_DBL;
ViBoolean dispErrFrIntrvl = DISP_DBL;
ViBoolean dispThresEI = DISP_DBL;
ViBoolean dispThresEFI = DISP_DBL;
ViBoolean dispErrPrfrm = DISP_DBL;

/*****************************************************************************/
/*= SET CONTROLLER CONDITIONS DURING READING PROCESS ========================*/
/*****************************************************************************/

ViInt16 headCntrl;
ViInt16 sdlCntrl;

/* Memorize Current Conditions and Set Header Off and Data Separator Comma  */
ViStatus CVIFUNC setCntrl (ViSession instrSession)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar rdBuffer[BUFFER_SIZE];   
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "HED?", 4, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, rdBuffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;                                                           
	if (Scan (rdBuffer, "%s>%d[b2]", &headCntrl) < 1)                 
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	    
	if ((avd3371_status = viWrite (instrSession, "SDL?", 4, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, rdBuffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;                                                           
	if (Scan (rdBuffer, "%s>%d[b2]", &sdlCntrl) < 1)               
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	    
	if ((avd3371_status = viWrite (instrSession, "HED 0;:SDL 0", 12, &retCount)) < 0)
        return avd3371_status;
    
    return avd3371_status;   
}

/* Reset GPIB Conditions */
ViStatus CVIFUNC resetCntrl (ViSession instrSession)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar wrtBuffer[BUFFER_SIZE]; 
    ViUInt32 retCount = 0;
    
    Fmt (wrtBuffer, "%s<HED %d;:SDL %d", headCntrl, sdlCntrl);
    
    if ((avd3371_status = viWrite (instrSession, wrtBuffer, NumFmtdBytes(), &retCount)) < 0)
        return avd3371_status;
    
    return avd3371_status;
} 

/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT STATUS/RANGE STRUCTURE  ============================*/
/*****************************************************************************/
/* avd3371_stringValPair is used in the avd3371_errorMessage function        */
/*===========================================================================*/

#define MagDisplay	   0
#define GrpDlyDisplay  2
#define CDDisplay	   3
#define CdSlpDisplay   4
#define PMDDisplay	   5

#define LogScale	   0
#define LinScale	   1

typedef struct {
	ViStatus stringVal;
	ViString stringName;
} avd3371_tStringValPair;

typedef struct {
	ViInt16 displayMode;
	ViInt16 levelScale;
} avd3371_status;

avd3371_status  instrStatus;

/*****************************************************************************/
/*= UTILITY ROUTINE DECLARATIONS (Non-Exportable Functions) =================*/
/*****************************************************************************/
ViBoolean avd3371_invalidViBooleanRange (ViBoolean val);
ViBoolean avd3371_invalidViInt16Range (ViInt16 val, ViInt16 min, ViInt16 max);
ViBoolean avd3371_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max);
ViBoolean avd3371_invalidViUInt8Range (ViUInt8 val, ViUInt8 min, ViUInt8 max);
ViBoolean avd3371_invalidViUInt16Range (ViUInt16 val, ViUInt16 min, ViUInt16 max);
ViBoolean avd3371_invalidViUInt32Range (ViUInt32 val, ViUInt32 min, ViUInt32 max);
ViBoolean avd3371_invalidViReal32Range (ViReal32 val, ViReal32 min, ViReal32 max);
ViBoolean avd3371_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max);
ViStatus  avd3371_initCleanUp (ViSession openRMSession, ViPSession openInstrSession,
								ViStatus currentStatus);

/*****************************************************************************/
/*====== USER-CALLABLE FUNCTIONS (Exportable Functions) =====================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Initialize                                                      */
/* Purpose:  This function opens the instrument, queries the instrument      */
/*           for its ID, and initializes the instrument to a known state.    */
/*===========================================================================*/
ViStatus _VI_FUNC avd3371_Initialize (ViRsrc resourceName, ViBoolean IDQuery,
                                       ViBoolean resetDevice, ViSession *instrSession)
{
    ViStatus avd3371_status = VI_SUCCESS;
    ViSession rmSession = 0;
    ViUInt32 retCnt = 0;
    ViByte rdBuffer[BUFFER_SIZE];

    /*- Check input parameter ranges ----------------------------------------*/
    if (avd3371_invalidViBooleanRange (IDQuery))
        return VI_ERROR_PARAMETER2;
    if (avd3371_invalidViBooleanRange (resetDevice))
        return VI_ERROR_PARAMETER3;

    /*- Open instrument session ---------------------------------------------*/
    if ((avd3371_status = viOpenDefaultRM (&rmSession)) < 0)
        return avd3371_status;

    if ((avd3371_status = viOpen (rmSession, resourceName, VI_NULL, VI_NULL, instrSession)) < 0) {
        viClose (rmSession);
        return avd3371_status;
    }

    /*- Configure VISA Formatted I/O ----------------------------------------*/
    if ((avd3371_status = viSetAttribute (*instrSession, VI_ATTR_TMO_VALUE, 10000)) < 0)
            return avd3371_initCleanUp (rmSession, instrSession, avd3371_status);
  
    if ((avd3371_status = viSetBuf (*instrSession, VI_READ_BUF|VI_WRITE_BUF, 4000)) < 0)
            return avd3371_initCleanUp (rmSession, instrSession, avd3371_status);
  
    if ((avd3371_status = viSetAttribute (*instrSession, VI_ATTR_WR_BUF_OPER_MODE,
                            VI_FLUSH_ON_ACCESS)) < 0)
            return avd3371_initCleanUp (rmSession, instrSession, avd3371_status);
    
    if ((avd3371_status = viSetAttribute (*instrSession, VI_ATTR_RD_BUF_OPER_MODE,
                            VI_FLUSH_ON_ACCESS)) < 0)
            return avd3371_initCleanUp (rmSession, instrSession, avd3371_status);

    /*- Identification Query ------------------------------------------------*/
    if (IDQuery) 
    {
        if ((avd3371_status = viWrite (*instrSession, "*IDN?", 5, &retCnt)) < 0)
            return avd3371_initCleanUp (rmSession, instrSession, avd3371_status);
        if ((avd3371_status = viRead (*instrSession, rdBuffer, BUFFER_SIZE, &retCnt)) < 0)
            return avd3371_status;

        Scan (rdBuffer, "ADVANTEST,D3371");
        if (NumFmtdBytes () != 15) 
          	return avd3371_initCleanUp (rmSession, instrSession, VI_ERROR_FAIL_ID_QUERY);
    }
        
    /*- Reset instrument ----------------------------------------------------*/
    if (resetDevice)
    {
        if ((avd3371_status = avd3371_Reset (*instrSession)) < 0)
            return avd3371_initCleanUp (rmSession, instrSession, avd3371_status);
    
    	dispBitCnt = DISP_DBL;
   		dispErrCnt = DISP_DBL;
    	dispImmErrCnt = DISP_DBL;
    	dispErrIntrvl = DISP_DBL;
    	dispErrFrIntrvl = DISP_DBL;
    	dispThresEI = DISP_DBL;
    	dispThresEFI = DISP_DBL;
    	dispErrPrfrm = DISP_DBL;
    }       
          
    return avd3371_status;
}

/*===========================================================================*/
/* Function: Set SSG Module                                                  */
/* Purpose:  This function sets the parameters of the 3.6GHz synthesizer	 */
/*			 module (SSG module). 											 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Set_SSGModule (ViSession instrSession, ViBoolean clkOut,
                                        ViInt32 clkFreq, ViBoolean clkRef)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avd3371_invalidViBooleanRange (clkOut))
        return VI_ERROR_PARAMETER2;
    if (avd3371_invalidViInt32Range (clkFreq, 10000, 3800000))
        return VI_ERROR_PARAMETER3;
    if (avd3371_invalidViBooleanRange (clkRef))
        return VI_ERROR_PARAMETER4;
    
    Fmt (wrtBuffer, "%s<SOUR10:FREQ %d;:SOUR10:REF %s;:SOUR10:OUTP %d", clkFreq, sigSrcArr[clkRef], clkOut);
    
    if ((avd3371_status = viWrite (instrSession, wrtBuffer, NumFmtdBytes(), &retCount)) < 0)
        return avd3371_status;
    
    return avd3371_status;    
}

/*===========================================================================*/
/* Function: Set PPG Pattern Type                                            */
/* Purpose:  This function sets data pattern type outputted from the pulse   */
/*			 pattern generator module (PPG module).							 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Set_PPG_PatternType (ViSession instrSession,
                                              ViInt16 ptrType, ViInt16 lenPRBS,
                                              ViInt16 mkrRtoPRBS, ViInt16 lenZSUB,
                                              ViInt16 lenCntZero, ViInt32 lenPROG,
                                              ViInt32 strBit, ViString ptrData, 
                                              ViBoolean pol)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar wrtBuffer[BUFFER_SIZE];
	ViInt16 n;
    ViUInt32 retCount = 0;
	ViUInt32 total = 0;
	ViChar *ptPtrData;
	
    if (avd3371_invalidViInt16Range (ptrType, 0, 2))
        return VI_ERROR_PARAMETER2;
        
    if (ptrType==0)  /* PRBS Pattern */
    {
    	if (avd3371_invalidViInt16Range (lenPRBS, 7, 31))
        	return VI_ERROR_PARAMETER3;
    	if (avd3371_invalidViInt16Range (mkrRtoPRBS, 0, 7))
        	return VI_ERROR_PARAMETER4;
        
        if (lenPRBS==8)
        	lenPRBS = 9;
        else if (lenPRBS>=12 && lenPRBS<=14)
        	lenPRBS = 15;
        else if (lenPRBS>=16 && lenPRBS<=22)
        	lenPRBS = 23;
        else if (lenPRBS>=24 && lenPRBS<=30)
        	lenPRBS = 31;
    }
    else if (ptrType==1)  /* ZSUB Pattern */
    {
    	if (avd3371_invalidViInt16Range (lenZSUB, 7, 15))
        	return VI_ERROR_PARAMETER5;
    	
        if (lenZSUB==8)
        	lenZSUB = 9;
        else if (lenZSUB>=12 && lenZSUB<=14)
        	lenZSUB = 15;
    	
    	switch (lenZSUB)
    	{
    	case 7:
    		n = 127;
    		break;
    	case 9:
    		n = 511;
    		break;
    	case 10:
    		n = 1023;
    		break;
    	case 11:
    		n = 2047;
    		break;
    	case 15:
    		n =32767;
    		break;
    	}
    	if (avd3371_invalidViInt16Range (lenCntZero, lenZSUB, n))
        	return VI_ERROR_PARAMETER6;
    }
    
    else if (ptrType==2)   /* PROG Pattern */
    {
    	if (avd3371_invalidViInt32Range (lenPROG, 1, 8388608))
        	return VI_ERROR_PARAMETER7;
    	if (avd3371_invalidViInt32Range (strBit, 0, lenPROG-1))
        	return VI_ERROR_PARAMETER8;
    }
        
    if (avd3371_invalidViBooleanRange (pol))
        return VI_ERROR_INSTR_PARAMETER10;
        
    if (ptrType==0)  /* PRBS Pattern */
    {
    	Fmt (wrtBuffer, "%s<SOUR:PATT PRBS%d;:SOUR:PATT:PRBS:MRAT %s", lenPRBS, mkrRtoArr[mkrRtoPRBS]);
    	total += NumFmtdBytes();
    }
    else if (ptrType==1)  /* ZSUB Pattern */
    {
    	Fmt (wrtBuffer, "%s<SOUR:PATT ZSUB%d;:SOUR:PATT:ZSUB %d", lenZSUB, lenCntZero);
    	total += NumFmtdBytes();
    }
    else if (ptrType==2)   /* PROG Pattern */
    {
    	Fmt (wrtBuffer, "%s<SOUR:PATT PROG;:SOUR:PATT:PROG %d", lenPROG);
    	total += NumFmtdBytes();
    	
    	n = strlen (ptrData);
    	ptPtrData = ptrData;
    	while (n>0)
    	{
    		if (n>512)
    		{
    			Fmt (wrtBuffer, "%s[a]<;:SOUR:PATT:PROG:DATA %d,2048,%cH%s%c", strBit, ptPtrData);
    			total += NumFmtdBytes();
    			
    			n -= 512;
    			strBit += 2048;
    			ptPtrData += 512;
    		}
    		
    		else
    		{
    			Fmt (wrtBuffer, "%s[a]<;:SOUR:PATT:PROG:DATA %d,%d,%cH%s%c", strBit, n*4, 34, ptPtrData, 34);
    			total += NumFmtdBytes();
    			
    			n = 0;
    		}
    	}
    }
    
    Fmt (wrtBuffer, "%s[a]<;:SOUR:PATT:POL %s", polArr[pol]);
    total += NumFmtdBytes();
        
    if ((avd3371_status = viWrite (instrSession, wrtBuffer, total, &retCount)) < 0)
        return avd3371_status;
    
    return avd3371_status;
}

/*===========================================================================*/
/* Function: Add Errors to PPG Pattern                                       */
/* Purpose:  This function controls to add errors to the data pattern of the */
/*			 PPG module.  													 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_AddErr_PPGPattern (ViSession instrSession,
                                            ViBoolean addErr, ViInt16 addMode,
                                            ViReal64 addRate, ViInt16 addRoute)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
	ViUInt32 total = 0;
    
    if (avd3371_invalidViBooleanRange (addErr))
        return VI_ERROR_PARAMETER2;
        
    if (addErr==VI_ON)
    {
    	if (avd3371_invalidViInt16Range (addMode, 0, 2))
        	return VI_ERROR_PARAMETER3;
    	if (avd3371_invalidViInt16Range (addRoute, 1, 16))
        	return VI_ERROR_PARAMETER5;
    	
    	if (addMode==1)
    		if (avd3371_invalidViReal64Range (addRate, 1.0e-9, 1.0e-2))
        		return VI_ERROR_PARAMETER4;
    }
    
    if (addErr==VI_ON)
    {
    	Fmt (wrtBuffer, "%s<SOUR:EADD 1;:SOUR:EADD:MODE %s;:SOUR:EADD:ROUT %d", addErrModeArr[addMode], addRoute);
    	total += NumFmtdBytes();
    	
    	if (addMode==1)
    	{
    		Fmt (wrtBuffer, "%s[a]<;:SOUR:EADD:RATE %f[e]", addRate);
    		total += NumFmtdBytes();
    	}
    }
    else
    {
    	Fmt (wrtBuffer, "%s<SOUR:EADD 0");
    	total += NumFmtdBytes();
    }

    if ((avd3371_status = viWrite (instrSession, wrtBuffer, total, &retCount)) < 0)
        return avd3371_status;
    
    return avd3371_status;
}

/*===========================================================================*/
/* Function: Set Clock Delay                                                 */
/* Purpose:  This function sets the clock delay.							 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Set_ClockDelay (ViSession instrSession, ViInt16 clkDly)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
	
    if (avd3371_invalidViInt16Range (clkDly, -1000, 1000))
   		return VI_ERROR_PARAMETER2;
   		
   	Fmt (wrtBuffer, "%s<SOUR2:DEL %d", clkDly);	

    if ((avd3371_status = viWrite (instrSession, wrtBuffer, NumFmtdBytes(), &retCount)) < 0)
        return avd3371_status;
    
    return avd3371_status;
}

/*===========================================================================*/
/* Function: Set Data Output                                                 */
/* Purpose:  This function sets the Data outputting parameters.				 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Set_DataOutput (ViSession instrSession,
                                         ViBoolean dataOut, ViReal64 amp, 
                                         ViInt16 ofstMode, ViReal64 ofstVal,
                                         ViInt16 crossPnt, ViInt16 termVol)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    ViReal64 offset;
	
    if (avd3371_invalidViBooleanRange (dataOut))
   		return VI_ERROR_PARAMETER2;
    if (avd3371_invalidViInt16Range (ofstMode, 0, 2))
   		return VI_ERROR_PARAMETER4;
    if (avd3371_invalidViInt16Range (crossPnt, 20, 80))
   		return VI_ERROR_PARAMETER6;
    if (avd3371_invalidViInt16Range (termVol, 0, 2))
   		return VI_ERROR_PARAMETER7;
   	
   	switch (ofstMode)
   	{
   	case 0:   /* High Level Offset */
   		offset = ofstVal;
   		break;
   		
   	case 1:   /* Middle Level Offset */
   		offset = ofstVal + amp / 2.0;
   		break;
   		
   	case 2:   /* Low Level Offset */
   		offset = ofstVal + amp;
   		break;
   	}
   		
   	switch (termVol)
   	{
   	case 0:   /* +1.3V */
    	if (avd3371_invalidViReal64Range (amp, 0.60, 1.00))
   			return VI_ERROR_PARAMETER3;
    	if (avd3371_invalidViReal64Range (offset, 2.30, 2.70))
   			return VI_ERROR_PARAMETER5;
   		break;
   		
   	case 1:   /* 0V */
    	if (avd3371_invalidViReal64Range (amp, 0.50, 3.00))
   			return VI_ERROR_PARAMETER3;
    	if (avd3371_invalidViReal64Range (offset, -2.00, 2.00))
   			return VI_ERROR_PARAMETER5;
   		break;
   	
   	case 2:   /* -2V */
    	if (avd3371_invalidViReal64Range (amp, 0.60, 1.00))
   			return VI_ERROR_PARAMETER3;
    	if (avd3371_invalidViReal64Range (offset, -1.00, -0.60))
   			return VI_ERROR_PARAMETER5;
   		break;
   	}
   	
   	if (dataOut==VI_ON)
   		Fmt (wrtBuffer, "%s<OUTP 1;:OUTP:TERM %s;:OUTP:VOLT %f;:OUTP:VOLT:%s %f;:OUTP:CROS %d",
   			 termVoltOutArr[termVol], amp, offsetModeArr[ofstMode], ofstVal, crossPnt);
   	else
   		Fmt (wrtBuffer, "%s<OUTP 0");
   	
    if ((avd3371_status = viWrite (instrSession, wrtBuffer, NumFmtdBytes(), &retCount)) < 0)
        return avd3371_status;
    
   	return avd3371_status;
}

/*===========================================================================*/
/* Function: Set XData Output                                                */
/* Purpose:  This function sets the XData outputting parameters.			 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Set_XDataOutput (ViSession instrSession,
										  ViBoolean track, ViBoolean xdataOut, 
                                          ViReal64 amp, ViInt16 ofstMode,
                                          ViReal64 ofstVal, ViInt16 crossPnt,
                                          ViInt16 termVol)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    ViReal64 offset;
	
    if (avd3371_invalidViBooleanRange (track))
   		return VI_ERROR_PARAMETER2;
    if (avd3371_invalidViBooleanRange (xdataOut))
   		return VI_ERROR_PARAMETER3;
    if (avd3371_invalidViInt16Range (ofstMode, 0, 2))
   		return VI_ERROR_PARAMETER5;
    if (avd3371_invalidViInt16Range (crossPnt, 20, 80))
   		return VI_ERROR_PARAMETER7;
    if (avd3371_invalidViInt16Range (termVol, 0, 2))
   		return VI_ERROR_PARAMETER8;
   	
   	switch (ofstMode)
   	{
   	case 0:   /* High Level Offset */
   		offset = ofstVal;
   		break;
   		
   	case 1:   /* Middle Level Offset */
   		offset = ofstVal + amp / 2.0;
   		break;
   		
   	case 2:   /* Low Level Offset */
   		offset = ofstVal + amp;
   		break;
   	}
   		
   	switch (termVol)
   	{
   	case 0:   /* +1.3V */
    	if (avd3371_invalidViReal64Range (amp, 0.60, 1.00))
   			return VI_ERROR_PARAMETER4;
    	if (avd3371_invalidViReal64Range (offset, 2.30, 2.70))
   			return VI_ERROR_PARAMETER6;
   		break;
   		
   	case 1:   /* 0V */
    	if (avd3371_invalidViReal64Range (amp, 0.50, 3.00))
   			return VI_ERROR_PARAMETER4;
    	if (avd3371_invalidViReal64Range (offset, -2.00, 2.00))
   			return VI_ERROR_PARAMETER6;
   		break;
   	
   	case 2:   /* -2V */
    	if (avd3371_invalidViReal64Range (amp, 0.60, 1.00))
   			return VI_ERROR_PARAMETER4;
    	if (avd3371_invalidViReal64Range (offset, -1.00, -0.60))
   			return VI_ERROR_PARAMETER6;
   		break;
   	}
   	
   	if (track==VI_ON)
   		Fmt (wrtBuffer, "OUTP3:TRAC 1");
   	else if (xdataOut==VI_ON)
   		Fmt (wrtBuffer, "%s<OUTP3:TRAC 0;:OUTP3 1;:OUTP3:TERM %s;:OUTP3:VOLT %f;:OUTP3:VOLT:%s %f;:OUTP3:CROS %d",
   			 termVoltOutArr[termVol], amp, offsetModeArr[ofstMode], ofstVal, crossPnt);
   	else
   		Fmt (wrtBuffer, "%s<OUTP3:TRAC 0;:OUTP3 0");
   	
    if ((avd3371_status = viWrite (instrSession, wrtBuffer, NumFmtdBytes(), &retCount)) < 0)
        return avd3371_status;
    
   	return avd3371_status;
}

/*===========================================================================*/
/* Function: Set Clock Output                                                */
/* Purpose:  This function sets the Clock outputting parameters.			 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Set_ClockOutput (ViSession instrSession,
                                          ViBoolean clockOut, ViReal64 amp, 
                                          ViInt16 ofstMode, ViReal64 ofstVal,
                                          ViInt16 termVol)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    ViReal64 offset;
	
    if (avd3371_invalidViBooleanRange (clockOut))
   		return VI_ERROR_PARAMETER2;
    if (avd3371_invalidViInt16Range (ofstMode, 0, 2))
   		return VI_ERROR_PARAMETER4;
    if (avd3371_invalidViInt16Range (termVol, 0, 2))
   		return VI_ERROR_PARAMETER6;
   	
   	switch (ofstMode)
   	{
   	case 0:   /* High Level Offset */
   		offset = ofstVal;
   		break;
   		
   	case 1:   /* Middle Level Offset */
   		offset = ofstVal + amp / 2.0;
   		break;
   		
   	case 2:   /* Low Level Offset */
   		offset = ofstVal + amp;
   		break;
   	}
   		
   	switch (termVol)
   	{
   	case 0:   /* +1.3V */
    	if (avd3371_invalidViReal64Range (amp, 0.60, 1.00))
   			return VI_ERROR_PARAMETER3;
    	if (avd3371_invalidViReal64Range (offset, 2.30, 2.70))
   			return VI_ERROR_PARAMETER5;
   		break;
   		
   	case 1:   /* 0V */
    	if (avd3371_invalidViReal64Range (amp, 0.50, 2.00))
   			return VI_ERROR_PARAMETER3;
    	if (avd3371_invalidViReal64Range (offset, -2.00, 2.00))
   			return VI_ERROR_PARAMETER5;
   		break;
   	
   	case 2:   /* -2V */
    	if (avd3371_invalidViReal64Range (amp, 0.60, 1.00))
   			return VI_ERROR_PARAMETER3;
    	if (avd3371_invalidViReal64Range (offset, -1.00, -0.60))
   			return VI_ERROR_PARAMETER5;
   		break;
   	}
   	
   	if (clockOut==VI_ON)
   		Fmt (wrtBuffer, "%s<OUTP2 1;:OUTP2:TERM %s;:OUTP2:VOLT %f;:OUTP2:VOLT:%s %f",
   			 termVoltOutArr[termVol], amp, offsetModeArr[ofstMode], ofstVal);
   	else
   		Fmt (wrtBuffer, "%s<OUTP2 0");
   	
    if ((avd3371_status = viWrite (instrSession, wrtBuffer, NumFmtdBytes(), &retCount)) < 0)
        return avd3371_status;
    
   	return avd3371_status;
}

/*===========================================================================*/
/* Function: Set XClock Output                                               */
/* Purpose:  This function sets the XClock outputting parameters.			 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Set_XClockOutput (ViSession instrSession,
                                           ViBoolean track, ViBoolean xclockOut,
                                           ViReal64 amp, ViInt16 ofstMode,
                                           ViReal64 ofstVal, ViInt16 termVol)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    ViReal64 offset;
	
    if (avd3371_invalidViBooleanRange (track))
   		return VI_ERROR_PARAMETER2;
    if (avd3371_invalidViBooleanRange (xclockOut))
   		return VI_ERROR_PARAMETER3;
    if (avd3371_invalidViInt16Range (ofstMode, 0, 2))
   		return VI_ERROR_PARAMETER5;
    if (avd3371_invalidViInt16Range (termVol, 0, 2))
   		return VI_ERROR_PARAMETER7;
   	
   	switch (ofstMode)
   	{
   	case 0:   /* High Level Offset */
   		offset = ofstVal;
   		break;
   		
   	case 1:   /* Middle Level Offset */
   		offset = ofstVal + amp / 2.0;
   		break;
   		
   	case 2:   /* Low Level Offset */
   		offset = ofstVal + amp;
   		break;
   	}
   		
   	switch (termVol)
   	{
   	case 0:   /* +1.3V */
    	if (avd3371_invalidViReal64Range (amp, 0.60, 1.00))
   			return VI_ERROR_PARAMETER4;
    	if (avd3371_invalidViReal64Range (offset, 2.30, 2.70))
   			return VI_ERROR_PARAMETER6;
   		break;
   		
   	case 1:   /* 0V */
    	if (avd3371_invalidViReal64Range (amp, 0.50, 2.00))
   			return VI_ERROR_PARAMETER4;
    	if (avd3371_invalidViReal64Range (offset, -2.00, 2.00))
   			return VI_ERROR_PARAMETER6;
   		break;
   	
   	case 2:   /* -2V */
    	if (avd3371_invalidViReal64Range (amp, 0.60, 1.00))
   			return VI_ERROR_PARAMETER4;
    	if (avd3371_invalidViReal64Range (offset, -1.00, -0.60))
   			return VI_ERROR_PARAMETER6;
   		break;
   	}
   	
   	if (track==VI_ON)
   		Fmt (wrtBuffer, "%s<OUTP4:TRAC 1");
   	else if (xclockOut==VI_ON)
   		Fmt (wrtBuffer, "%s<OUTP4:TRAC 0;:OUTP4 1;:OUTP4:TERM %s;:OUTP4:VOLT %f;:OUTP4:VOLT:%s %f",
   			 termVoltOutArr[termVol], amp, offsetModeArr[ofstMode], ofstVal);
   	else
   		Fmt (wrtBuffer, "%s<OUTP4:TRAC 0;:OUTP4 0");
   	
    if ((avd3371_status = viWrite (instrSession, wrtBuffer, NumFmtdBytes(), &retCount)) < 0)
        return avd3371_status;
    
   	return avd3371_status;
}

/*===========================================================================*/
/* Function: Set Burst Output                                                */
/* Purpose:  This function sets parameters of the burst signal.				 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Set_BurstOutput (ViSession instrSession,
                                          ViBoolean bstMode, ViBoolean bstSrc,
                                          ViInt32 bstCycle, ViInt32 offTime)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avd3371_invalidViBooleanRange (bstMode))
   		return VI_ERROR_PARAMETER2;
    if (avd3371_invalidViBooleanRange (bstSrc))
   		return VI_ERROR_PARAMETER3;
    if (avd3371_invalidViInt32Range (bstCycle, 2, 50000))
   		return VI_ERROR_PARAMETER4;
    if (avd3371_invalidViInt32Range (offTime, 1, 1000))
   		return VI_ERROR_PARAMETER5;
    if (avd3371_invalidViInt32Range (offTime, 1, bstCycle))
   		return VI_ERROR_PARAMETER5;
   		
   	if (bstMode==VI_ON)
   		Fmt (wrtBuffer, "%s<SOUR5:BOUT 1;:SOUR5:BOUT:SOUR %s;:SOUR5:BOUT:OTIM 1;:SOUR5:BOUT:CYCL %d;:SOUR5:BOUT:OTIM %d",
   			 sigSrcArr[bstSrc], bstCycle, offTime);
   	else
   		Fmt (wrtBuffer, "%s<SOUR5:BOUT 0");
    
    if ((avd3371_status = viWrite (instrSession, wrtBuffer, NumFmtdBytes(), &retCount)) < 0)
        return avd3371_status;
    
   	return avd3371_status;
}

/*===========================================================================*/
/* Function: Set Trigger Output                                              */
/* Purpose:  This function sets the parameters of the trigger signal.		 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Set_TriggerOutput (ViSession instrSession,
                                            ViInt16 trgSrc, ViInt32 trgPosPRBS,
                                            ViInt32 trgPosZSUB, ViInt32 trgPosPROG)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avd3371_invalidViInt16Range (trgSrc, 0, 2))
   		return VI_ERROR_PARAMETER2;
   		
   	Fmt (wrtBuffer, "%s<SOUR6:TRIG %s;:SOUR6:TRIG:PRBS:POS %d;:SOUR6:TRIG:ZSUB:POS %d;:SOUR6:TRIG:PROG:POS %d",
   		 trgSrcArr[trgSrc], trgPosPRBS, trgPosZSUB, trgPosPROG);
    
    if ((avd3371_status = viWrite (instrSession, wrtBuffer, NumFmtdBytes(), &retCount)) < 0)
        return avd3371_status;
    
   	return avd3371_status;
}

/*===========================================================================*/
/* Function: Set ED Pattern Type                                             */
/* Purpose:  This function sets data pattern type outputted from the error   */
/*			 detector module (ED module).									 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Set_ED_PatternType (ViSession instrSession,
                                             ViBoolean track, ViInt16 ptrType,
                                             ViInt16 lenPRBS, ViInt16 mkrRtoPRBS,
                                             ViInt16 lenZSUB, ViInt16 lenCntZero,
                                             ViInt32 lenPROG, ViInt32 strBit,
                                             ViString ptrData, ViBoolean pol)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar wrtBuffer[BUFFER_SIZE];
	ViInt16 n;
    ViUInt32 retCount = 0;
	ViUInt32 total = 0;
	ViChar *ptPtrData;
	
    if (avd3371_invalidViBooleanRange (track))
        return VI_ERROR_PARAMETER2;
    if (avd3371_invalidViInt16Range (ptrType, 0, 2))
        return VI_ERROR_PARAMETER3;
        
    if (ptrType==0)  /* PRBS Pattern */
    {
    	if (avd3371_invalidViInt16Range (lenPRBS, 7, 31))
        	return VI_ERROR_PARAMETER4;
    	if (avd3371_invalidViInt16Range (mkrRtoPRBS, 0, 7))
        	return VI_ERROR_PARAMETER5;
        	
        if (lenPRBS==8)
        	lenPRBS = 9;
        else if (lenPRBS>=12 && lenPRBS<=14)
        	lenPRBS = 15;
        else if (lenPRBS>=16 && lenPRBS<=22)
        	lenPRBS = 23;
        else if (lenPRBS>=24 && lenPRBS<=30)
        	lenPRBS = 31;
    }
    else if (ptrType==1)  /* ZSUB Pattern */
    {
    	if (avd3371_invalidViInt16Range (lenZSUB, 7, 15))
        	return VI_ERROR_PARAMETER6;
    	
        if (lenZSUB==8)
        	lenZSUB = 9;
        else if (lenZSUB>=12 && lenZSUB<=14)
        	lenZSUB = 15;
    	
    	switch (lenZSUB)
    	{
    	case 7:
    		n = 127;
    		break;
    	case 9:
    		n = 511;
    		break;
    	case 10:
    		n = 1023;
    		break;
    	case 11:
    		n = 2047;
    		break;
    	case 15:
    		n =32767;
    		break;
    	}
    	if (avd3371_invalidViInt16Range (lenCntZero, lenZSUB, n))
        	return VI_ERROR_PARAMETER7;
    }
    
    else if (ptrType==2)   /* PROG Pattern */
    {
    	if (avd3371_invalidViInt32Range (lenPROG, 1, 8388608))
        	return VI_ERROR_PARAMETER8;
    	if (avd3371_invalidViInt32Range (strBit, 0, lenPROG-1))
        	return VI_ERROR_INSTR_PARAMETER9;
    }
        
    if (avd3371_invalidViBooleanRange (pol))
        return VI_ERROR_INSTR_PARAMETER11;
    
    if (track==VI_ON)
    {
    	Fmt (wrtBuffer, "%s<SENS:PATT:TRAC 1");
    	total += NumFmtdBytes();
    }
        
    else
    {
    	if (ptrType==0)  /* PRBS Pattern */
   		{
    		Fmt (wrtBuffer, "%s<SENS:PATT:TRAC 0;:SENS:PATT PRBS%d;:SENS:PATT:PRBS:MRAT %s", lenPRBS, mkrRtoArr[mkrRtoPRBS]);
    		total += NumFmtdBytes();
    	}
    	else if (ptrType==1)  /* ZSUB Pattern */
    	{
    		Fmt (wrtBuffer, "%s<SENS:PATT:TRAC 0;:SENS:PATT ZSUB%d;:SENS:PATT:ZSUB %d", lenZSUB, lenCntZero);
    		total += NumFmtdBytes();
    	}
    	else if (ptrType==2)   /* PROG Pattern */
    	{
    		Fmt (wrtBuffer, "%s<SENS:PATT:TRAC 0;:SENS:PATT PROG;:SENS:PATT:PROG %d", lenPROG);
    		total += NumFmtdBytes();
    	
    		n = strlen (ptrData);
    		ptPtrData = ptrData;
    		while (n>0)
    		{
    			if (n>512)
    			{
    				Fmt (wrtBuffer, "%s[a]<;:SENS:PATT:PROG:DATA %d,2048,%cH%s%c", strBit, 34, ptPtrData, 34);
    				total += NumFmtdBytes();
    				
    				n -= 512;
    				strBit += 2048;
    				ptPtrData += 512;
    			}
    			
    			else
    			{
    				Fmt (wrtBuffer, "%s[a]<;:SENS:PATT:PROG:DATA %d,%d,%cH%s%c", strBit, n*4, 34, ptPtrData, 34);
    				total += NumFmtdBytes();
    				
    				n = 0;
    			}
    		}
    	}
    
    	Fmt (wrtBuffer, "%s[a]<;:SENS:PATT:POL %s", polArr[pol]);
    	total += NumFmtdBytes();
    }	
        
    if ((avd3371_status = viWrite (instrSession, wrtBuffer, total, &retCount)) < 0)
        return avd3371_status;
    
    return avd3371_status;
}

/*===========================================================================*/
/* Function: Set Synchronization                                             */
/* Purpose:  This function sets the synchronization parameters. 			 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Set_Synchronization (ViSession instrSession,
                                              ViInt16 autoSync, ViInt32 syncAdrs,
                                              ViInt16 syncLen, ViBoolean autoThres,
                                              ViReal64 syncGainPRBS, ViReal64 syncLossPRBS,
                                              ViReal64 syncGainMEM, ViReal64 syncLossMEM)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
	ViUInt32 total = 0;
	
    if (avd3371_invalidViInt16Range (autoSync, 0, 2))
        return VI_ERROR_PARAMETER2;
    if (avd3371_invalidViInt16Range (syncLen, 8, 32))
        return VI_ERROR_PARAMETER4;
    if (avd3371_invalidViBooleanRange (autoThres))
        return VI_ERROR_PARAMETER5;
    if (avd3371_invalidViReal64Range (syncGainPRBS, 1.0e-7, 1.0e-2))
        return VI_ERROR_PARAMETER6;
    if (avd3371_invalidViReal64Range (syncLossPRBS, syncGainPRBS, 1.0e-2))
        return VI_ERROR_PARAMETER7;
    if (avd3371_invalidViReal64Range (syncGainMEM, 1.0e-12, 1.0e-2))
        return VI_ERROR_PARAMETER8;
    if (avd3371_invalidViReal64Range (syncLossMEM, syncGainMEM, 1.0e-2))
        return VI_ERROR_INSTR_PARAMETER9;
 
 	Fmt (wrtBuffer, "%s<SENS:SYNC %s;:SENS:SYNC:PATT:PROG:ADDR %d;:SENS:SYNC:PATT:LENG %d",
 		 autoSyncArr[autoSync], syncAdrs, syncLen);
 	total += NumFmtdBytes();
 	
 	if (autoThres==VI_ON)
 		Fmt (wrtBuffer, "%s[a]<;:SENS:SYNC:THR:AUTO 1");
 	else
 		Fmt (wrtBuffer, "%s[a]<;:SENS:SYNC:THR:AUTO 0;:SENS:SYNC:THR:PRBS:GAIN %f[e];:SENS:SYNC:THR:PRBS:LOSS %f[e];:SENS:SYNC:THR:MEM:GAIN %f[e];:SENS:SYNC:THR:MEM:LOSS %f[e]",
 			 syncGainPRBS, syncLossPRBS, syncGainMEM, syncLossMEM);
 	total += NumFmtdBytes();
 	
    if ((avd3371_status = viWrite (instrSession, wrtBuffer, total, &retCount)) < 0)
        return avd3371_status;
    
    return avd3371_status;
}

/*===========================================================================*/
/* Function: Set Data Input		                                             */
/* Purpose:  This function sets the data input parameters.		 			 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Set_ED_DataInput (ViSession instrSession,
                                           ViBoolean pol, ViReal64 threshold,
                                           ViInt16 termVol)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avd3371_invalidViBooleanRange (pol))
   		return VI_ERROR_PARAMETER2;
    if (avd3371_invalidViInt16Range (termVol, 0, 3))
   		return VI_ERROR_PARAMETER4;
    
    switch (termVol)
    {
    case 0:   /* +3V */
    	if (avd3371_invalidViReal64Range (threshold, 3.150, 4.250))
   			return VI_ERROR_PARAMETER3;
   		break;
   		
    case 1:   /* +1.3V */
    	if (avd3371_invalidViReal64Range (threshold, 1.450, 2.550))
   			return VI_ERROR_PARAMETER3;
   		break;
   		
 	case 2:   /* 0V */
    	if (avd3371_invalidViReal64Range (threshold, -2.040, 2.040))
   			return VI_ERROR_PARAMETER3;
   		break;
   	case 3:   /* -2V */	
    	if (avd3371_invalidViReal64Range (threshold, -1.850, -0.750))
   			return VI_ERROR_PARAMETER3;
   		break;
    }
    
    Fmt (wrtBuffer, "%s<INP:POL %s;:INP:TERM %s;:INP:TVOL %f",
    	 polArr[pol], termVoltInpArr[termVol], threshold);
    
    if ((avd3371_status = viWrite (instrSession, wrtBuffer, NumFmtdBytes(), &retCount)) < 0)
        return avd3371_status;
    
    return avd3371_status;
}

/*===========================================================================*/
/* Function: Set Clock Input	                                             */
/* Purpose:  This function sets the clock input parameters.		 			 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Set_ED_ClockInput (ViSession instrSession,
                               		     	ViBoolean pol, ViInt16 clkDly,
                                         	ViInt16 termVol)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avd3371_invalidViBooleanRange (pol))
   		return VI_ERROR_PARAMETER2;
    if (avd3371_invalidViInt16Range (clkDly, -1000, 1000))
   		return VI_ERROR_PARAMETER3;
    if (avd3371_invalidViInt16Range (termVol, 0, 3))
   		return VI_ERROR_PARAMETER4;
    
    Fmt (wrtBuffer, "%s<INP2:POL %s;:INP2:DEL %d;:INP2:TERM %s",
    	 polArr[pol], clkDly, termVoltInpArr[termVol]);
    
    if ((avd3371_status = viWrite (instrSession, wrtBuffer, NumFmtdBytes(), &retCount)) < 0)
        return avd3371_status;
    
    return avd3371_status;
}

/*===========================================================================*/
/* Function: Mask Measurement Route                                          */
/* Purpose:  This function switches for received data on each route to		 */
/*			 measure or not. Received data is converted to parallel data	 */
/*			 (1:16) by MUX or DEMUX, and each route data is masked or not	 */
/*			 from the measurement.											 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Mask_MeasRoute (ViSession instrSession,
                                         ViInt16 routeNum, ViBoolean mask)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avd3371_invalidViInt16Range (routeNum, 1, 16))
   		return VI_ERROR_PARAMETER2;
    if (avd3371_invalidViBooleanRange (mask))
   		return VI_ERROR_PARAMETER3;
   		
   	Fmt (wrtBuffer, "%s<SENS:MASK:ROUT%d %d", routeNum, mask);	
    
    if ((avd3371_status = viWrite (instrSession, wrtBuffer, NumFmtdBytes(), &retCount)) < 0)
        return avd3371_status;
    
    return avd3371_status;
}

/*===========================================================================*/
/* Function: Set Auto Search		                                         */
/* Purpose:  This function sets auto search parameters. Parameter which is   */
/*			 set to auto search, is automatically set to optimum value in    */
/*			 the measurement.												 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Set_AutoSearch (ViSession instrSession,
                                         ViBoolean clock, ViBoolean threshold,
                                         ViBoolean pattern)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avd3371_invalidViBooleanRange (clock))
   		return VI_ERROR_PARAMETER2;
    if (avd3371_invalidViBooleanRange (threshold))
   		return VI_ERROR_PARAMETER3;
    if (avd3371_invalidViBooleanRange (pattern))
   		return VI_ERROR_PARAMETER4;
    
    Fmt (wrtBuffer, "%s<SENS:ASE:CLOC %d;:SENS:ASE:TVOL %d;:SENS:ASE:PATT %d",
    	 clock, threshold, pattern);
    	 
    if ((avd3371_status = viWrite (instrSession, wrtBuffer, NumFmtdBytes(), &retCount)) < 0)
        return avd3371_status;
    
    return avd3371_status;
}

/*===========================================================================*/
/* Function: Set LOG Report                                                  */
/* Purpose:  This function selects the contents for the LOG report.			 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Set_LOGReport (ViSession instrSession,
                                        ViBoolean basicMeas, ViBoolean lossHist,
                                        ViBoolean errRateHist, ViBoolean errCntHist,
                                        ViBoolean errIntrvl, ViBoolean errFrIntrvl,
                                        ViBoolean freqHist)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avd3371_invalidViBooleanRange (basicMeas))
   		return VI_ERROR_PARAMETER2;
    if (avd3371_invalidViBooleanRange (lossHist))
   		return VI_ERROR_PARAMETER3;
    if (avd3371_invalidViBooleanRange (errRateHist))
   		return VI_ERROR_PARAMETER4;
    if (avd3371_invalidViBooleanRange (errCntHist))
   		return VI_ERROR_PARAMETER5;
    if (avd3371_invalidViBooleanRange (errIntrvl))
   		return VI_ERROR_PARAMETER6;
    if (avd3371_invalidViBooleanRange (errFrIntrvl))
   		return VI_ERROR_PARAMETER7;
    if (avd3371_invalidViBooleanRange (freqHist))
   		return VI_ERROR_PARAMETER8;
   		
   	Fmt (wrtBuffer, "%s<SENS:LOG:BME %d;:SENS:LOG:HIST:SCL %d;:SENS:LOG:HIST:MDAT:IER %d;:SENS:LOG:HIST:MDAT:IEC %d;:SENS:LOG:HIST:MDAT:EINT %d;:SENS:LOG:HIST:MDAT:EFIN %d;:SENS:LOG:HIST:MDAT:FREQ %d",	
   		 basicMeas, lossHist, errRateHist, errCntHist, errIntrvl, errFrIntrvl, freqHist);
    
    if ((avd3371_status = viWrite (instrSession, wrtBuffer, NumFmtdBytes(), &retCount)) < 0)
        return avd3371_status;
    
    return avd3371_status;
}

/*===========================================================================*/
/* Function: Set Measurement Condition                                       */
/* Purpose:  This function sets measurement condition.						 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Set_MeasCondition (ViSession instrSession,
                                            ViInt16 timerMode, ViBoolean interval,
                                            ViInt16 periodDay, ViInt16 periodHour,
                                            ViInt16 periodMin, ViInt16 periodSec,
                                            ViBoolean burstMode, ViBoolean msClkLs,
                                            ViBoolean msSyncLs, ViBoolean threshold,
                                            ViBoolean dispCurr)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avd3371_invalidViInt16Range (timerMode, 0, 2))
   		return VI_ERROR_PARAMETER2;
    if (avd3371_invalidViBooleanRange (interval))
   		return VI_ERROR_PARAMETER3;
    if (avd3371_invalidViInt16Range (periodDay, 0,99))
   		return VI_ERROR_PARAMETER4;
    if (avd3371_invalidViInt16Range (periodHour, 0, 23))
   		return VI_ERROR_PARAMETER5;
    if (avd3371_invalidViInt16Range (periodMin, 0, 59))
   		return VI_ERROR_PARAMETER6;
    if (avd3371_invalidViInt16Range (periodSec, 0, 59))
   		return VI_ERROR_PARAMETER7;
    if (avd3371_invalidViBooleanRange (burstMode))
   		return VI_ERROR_PARAMETER8;
    if (avd3371_invalidViBooleanRange (msClkLs))
   		return VI_ERROR_INSTR_PARAMETER9;
    if (avd3371_invalidViBooleanRange (msSyncLs))
   		return VI_ERROR_INSTR_PARAMETER10;
    if (avd3371_invalidViBooleanRange (threshold))
   		return VI_ERROR_INSTR_PARAMETER11;
    if (avd3371_invalidViBooleanRange (dispCurr))
   		return VI_ERROR_INSTR_PARAMETER12;
   		
   	Fmt (wrtBuffer, "%s<SENS:BME:MTIM:MODE %s;:SENS:BME:MTIM:INT %s;:SENS:BME:MTIM:PER %d,%d,%d,%d;:SENS:BME:BURS %d;:SENS:BME:EVAL:CLOS %d;:SENS:BME:EVAL:SLOS %d;:SENS:BME:EPER:THR %s;:SENS:BME:CDAT %d",
   		 timerModeArr[timerMode], msIntrvlArr[interval], periodDay, periodHour, periodMin, periodSec, 
   		 burstMode, msClkLs, msSyncLs, errRateThresArr[threshold], dispCurr);
    
    if ((avd3371_status = viWrite (instrSession, wrtBuffer, NumFmtdBytes(), &retCount)) < 0)
        return avd3371_status;
    
    return avd3371_status;
}

/*===========================================================================*/
/* Function: Set Trigger Source			                                     */
/* Purpose:  This function sets the source of the trigger output.			 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Set_TriggerSource (ViSession instrSession,
                                            ViInt16 trgSrc)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avd3371_invalidViInt16Range (trgSrc, 0, 2))
   		return VI_ERROR_PARAMETER2;
    
    Fmt (wrtBuffer, "%s<SOUR8:TRIG %s", trgOutSrcArr[trgSrc]);
    
    if ((avd3371_status = viWrite (instrSession, wrtBuffer, NumFmtdBytes(), &retCount)) < 0)
        return avd3371_status;
    
    return avd3371_status;
}

/*===========================================================================*/
/* Function: Measurement				                                     */
/* Purpose:  This function switches the basic measurement.					 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Measurement (ViSession instrSession,
                                      ViBoolean measure)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avd3371_invalidViBooleanRange (measure))
   		return VI_ERROR_PARAMETER2;
    
    Fmt (wrtBuffer, "%s<SENS:BME %d", measure);
    
    if ((avd3371_status = viWrite (instrSession, wrtBuffer, NumFmtdBytes(), &retCount)) < 0)
        return avd3371_status;
    
    return avd3371_status;
}

/*===========================================================================*/
/* Function: Execute Auto Search		                                     */
/* Purpose:  This function switches auto search. When auto search is set to  */
/*			 On, selected parameters are automatically set to optimum value  */
/*			 in the measurement.											 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Exe_AutoSearch (ViSession instrSession,
                                         ViBoolean autoSrch)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avd3371_invalidViBooleanRange (autoSrch))
   		return VI_ERROR_PARAMETER2;
    
    Fmt (wrtBuffer, "%s<SENS:ASE %d", autoSrch);
    
    if ((avd3371_status = viWrite (instrSession, wrtBuffer, NumFmtdBytes(), &retCount)) < 0)
        return avd3371_status;
    
    return avd3371_status;
}

/*===========================================================================*/
/* Function: Display Format				                                     */
/* Purpose:  This function sets the display formats for measurement results. */
/*			 Display formats are effective both on the instrument's screen   */
/*			 and query values by GPIB.										 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_DisplayFormat (ViSession instrSession,
                                        ViBoolean bitCnt, ViBoolean errCnt,
                                        ViBoolean immErrCnt, ViBoolean errIntrvl,
                                        ViBoolean errFrIntrvl, ViBoolean thresEI,
                                        ViBoolean thresEFI, ViBoolean errPrfrm)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avd3371_invalidViBooleanRange (bitCnt))
   		return VI_ERROR_PARAMETER2;
    if (avd3371_invalidViBooleanRange (errCnt))
   		return VI_ERROR_PARAMETER3;
    if (avd3371_invalidViBooleanRange (immErrCnt))
   		return VI_ERROR_PARAMETER4;
    if (avd3371_invalidViBooleanRange (errIntrvl))
   		return VI_ERROR_PARAMETER5;
    if (avd3371_invalidViBooleanRange (errFrIntrvl))
   		return VI_ERROR_PARAMETER6;
    if (avd3371_invalidViBooleanRange (thresEI))
   		return VI_ERROR_PARAMETER7;
    if (avd3371_invalidViBooleanRange (thresEFI))
   		return VI_ERROR_PARAMETER8;
    if (avd3371_invalidViBooleanRange (errPrfrm))
   		return VI_ERROR_INSTR_PARAMETER9;
   		
   	Fmt (wrtBuffer, "%s<DISP:WIND:BCO:DFOR %s;:DISP:WIND:ECO:DFOR %s;:DISP:WIND:IEC:DFOR %s;:DISP:WIND:EINT:DFOR %s;:DISP:WIND:EFIN:DFOR %s;:DISP:WIND:TEIN:DFOR %s;:DISP:WIND:TEF:DFOR %s;:DISP:WIND:EPER:DFOR %s",
   		 frmtIntExpArr[bitCnt], frmtIntExpArr[errCnt], frmtIntExpArr[immErrCnt], frmtIntPrcArr[errIntrvl],
   		 frmtIntPrcArr[errFrIntrvl], frmtIntPrcArr[thresEI], frmtIntPrcArr[thresEFI], frmtIntPrcArr[errPrfrm]);
    
    if ((avd3371_status = viWrite (instrSession, wrtBuffer, NumFmtdBytes(), &retCount)) < 0)
        return avd3371_status;
        
    dispBitCnt = bitCnt;
    dispErrCnt = errCnt;
    dispImmErrCnt = immErrCnt;
    dispErrIntrvl = errIntrvl;
    dispErrFrIntrvl = errFrIntrvl;
    dispThresEI = thresEI;
    dispThresEFI = thresEFI;
    dispErrPrfrm = errPrfrm;
    
    return avd3371_status;
}

/*===========================================================================*/
/* Function: Print						                                     */
/* Purpose:  This function prints basic measurement results.				 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Print (ViSession instrSession)
{
	ViStatus avd3371_status = VI_SUCCESS;
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "HCOP:ED:BME", 11, &retCount)) < 0)
        return avd3371_status;
    
    return avd3371_status;
}

/*===========================================================================*/
/* Function: Load						                                     */
/* Purpose:  This function loads setting conditions etc. from file to		 */
/*			 instrument.													 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Load (ViSession instrSession,
							   ViInt16 kind, ViString fileName)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avd3371_invalidViInt16Range (kind, 0, 5))
   		return VI_ERROR_PARAMETER2;
    
    Fmt (wrtBuffer, "%s<%s %c%s%c", loadArr[kind], 34, fileName, 34);
    
    if ((avd3371_status = viWrite (instrSession, wrtBuffer, NumFmtdBytes(), &retCount)) < 0)
        return avd3371_status;
    
    return avd3371_status;
}

/*===========================================================================*/
/* Function: Save						                                     */
/* Purpose:  This function savess instrument's setting conditions etc. to    */
/*			 the file.														 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Save (ViSession instrSession, 
							   ViInt16 kind, ViString fileName)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avd3371_invalidViInt16Range (kind, 0, 6))
   		return VI_ERROR_PARAMETER2;
    
    Fmt (wrtBuffer, "%s<%s %c%s%c", saveArr[kind], 34, fileName, 34);
    
    if ((avd3371_status = viWrite (instrSession, wrtBuffer, NumFmtdBytes(), &retCount)) < 0)
        return avd3371_status;
    
    return avd3371_status;
}

/*===========================================================================*/
/* Function: Beep						                                     */
/* Purpose:  This function switches to beeep for data error or alarm 		 */
/*			 occured.														 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Beep (ViSession instrSession, 
							   ViBoolean beepError, ViBoolean beepAlarm)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avd3371_invalidViBooleanRange (beepError))
   		return VI_ERROR_PARAMETER2;
    if (avd3371_invalidViBooleanRange (beepAlarm))
   		return VI_ERROR_PARAMETER3;
    
    Fmt (wrtBuffer, "%s<SYST:BEEP:DERR %d;:SYST:BEEP:ALAR %d", beepError, beepAlarm);
    
    if ((avd3371_status = viWrite (instrSession, wrtBuffer, NumFmtdBytes(), &retCount)) < 0)
        return avd3371_status;
    
    return avd3371_status;
}

/*===========================================================================*/
/* Function: Set Date and Time			                                     */
/* Purpose:  This function sets the instrument's clock.				 		 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Set_DateTime (ViSession instrSession,
									   ViInt16 year, ViInt16 month, 
									   ViInt16 day, ViInt16 hour,
                                       ViInt16 minute)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avd3371_invalidViInt16Range (year, 1980, 2099))
   		return VI_ERROR_PARAMETER2;
    if (avd3371_invalidViInt16Range (month, 1, 12))
   		return VI_ERROR_PARAMETER3;
    if (avd3371_invalidViInt16Range (day, 1, 31))
   		return VI_ERROR_PARAMETER4;
    if (avd3371_invalidViInt16Range (hour, 0, 23))
   		return VI_ERROR_PARAMETER5;
    if (avd3371_invalidViInt16Range (minute, 0, 59))
   		return VI_ERROR_PARAMETER6;
   		
   	Fmt (wrtBuffer, "%s<SYST:DATE %d,%d,%d;:SYST:TIME %d,%d,0",
   		 year, month, day, hour, minute);
    
    if ((avd3371_status = viWrite (instrSession, wrtBuffer, NumFmtdBytes(), &retCount)) < 0)
        return avd3371_status;
    
    return avd3371_status;
}

/*===========================================================================*/
/* Function: Set Standard Event Status Enable Register                       */
/* Purpose:  This function sets the standard event status enable register.	 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Set_StandardEventEnblRgstr (ViSession instrSession,
                                                     ViInt32 value)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avd3371_invalidViInt16Range (value, 0, 255))
   		return VI_ERROR_PARAMETER2;
    
    Fmt (wrtBuffer, "%s<*ESE %d", value);
    
    if ((avd3371_status = viWrite (instrSession, wrtBuffer, NumFmtdBytes(), &retCount)) < 0)
        return avd3371_status;
    
    return avd3371_status;
}

/*===========================================================================*/
/* Function: Read Standard Event Status Register                             */
/* Purpose:  This function reads standard event status register.			 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Read_StandardEvntRgstr (ViSession instrSession,
                                                 ViInt32 *value)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "*ESR?", 5, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;                                                           
	if (Scan (buffer, "%s>%d", value) < 1)               
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	    
	return avd3371_status;   
}

/*===========================================================================*/
/* Function: Set Service Request Enable Register		                     */
/* Purpose:  This function sets the service request enable register.		 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Set_ServiceReqEnblRgstr (ViSession instrSession,
                                                  ViInt32 value)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avd3371_invalidViInt16Range (value, 0, 255))
   		return VI_ERROR_PARAMETER2;
    
    Fmt (wrtBuffer, "%s<*SRE %d", value);
    
    if ((avd3371_status = viWrite (instrSession, wrtBuffer, NumFmtdBytes(), &retCount)) < 0)
        return avd3371_status;
    
    return avd3371_status;
}

/*===========================================================================*/
/* Function: Read Service Request Enable Register                  			 */
/* Purpose:  This function reads service request enable register.			 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Read_ServiceReqEnblRgstr (ViSession instrSession,
                                                   ViInt32 *value)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "*SRE?", 5, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;                                                           
	if (Scan (buffer, "%s>%d", value) < 1)               
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	    
	return avd3371_status;   
}

/*===========================================================================*/
/* Function: Read Status Byte Register                          			 */
/* Purpose:  This function reads status byte register.						 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Read_STB (ViSession instrSession,
                                   ViInt32 *value)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "*STB?", 5, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;                                                           
	if (Scan (buffer, "%s>%d32", value) < 1)               
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	    
	return avd3371_status;   
}

/*===========================================================================*/
/* Function: Set Operation Status Enable Register                            */
/* Purpose:  This function sets the operation status enable register.		 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Set_OperationEnblRgstr (ViSession instrSession,
                                      			 ViInt32 value)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avd3371_invalidViInt32Range (value, 0, 65535))
   		return VI_ERROR_PARAMETER2;
    
    Fmt (wrtBuffer, "%s<STAT:OPER:ENAB %d", value);
    
    if ((avd3371_status = viWrite (instrSession, wrtBuffer, NumFmtdBytes(), &retCount)) < 0)
        return avd3371_status;
    
    return avd3371_status;
}

/*===========================================================================*/
/* Function: Wait										                     */
/* Purpose:  This function waits for executing command to end. Using this	 */
/*			 wait function, commands after this function are delayed, till   */
/*			 executing command ends.										 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Wait (ViSession instrSession)
{
	ViStatus avd3371_status = VI_SUCCESS;
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "*WAI", 4, &retCount)) < 0)
        return avd3371_status;
    
    return avd3371_status;
}

/*===========================================================================*/
/* Function: Clear Status Bytes							                     */
/* Purpose:  This function clears status bytes and data concentrated with	 */
/*			 "*OPC" and "*OPC?".											 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Clear_StatusBytes (ViSession instrSession)
{
    ViStatus avd3371_status = VI_SUCCESS;
	ViUInt32 retCount = 0; 

    if ((avd3371_status = viWrite (instrSession, "*CLS", 4, &retCount)) < 0)
        return avd3371_status;
    
    return avd3371_status;
}

/*===========================================================================*/
/* Function: Set Transition Filter of Operation Status                       */
/* Purpose:  This function sets the transition filter of the operation 		 */
/*			 status.														 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Set_OperationTrnsFltr (ViSession instrSession,
                                      	        ViBoolean direct, ViInt32 value)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avd3371_invalidViBooleanRange (direct))
   		return VI_ERROR_PARAMETER2;
    if (avd3371_invalidViInt32Range (value, 0, 65535))
   		return VI_ERROR_PARAMETER3;
   		
   	Fmt (wrtBuffer, "%s<STAT:OPER:%s %d", directArr[direct], value);	
    
    if ((avd3371_status = viWrite (instrSession, wrtBuffer, NumFmtdBytes(), &retCount)) < 0)
        return avd3371_status;
    
    return avd3371_status;
}

/*===========================================================================*/
/* Function: Read Operation Status Event Register                            */
/* Purpose:  This function reads operation status event register. When this  */
/*			 function is executed, this register is cleared.				 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Read_OperationEvntRgstr (ViSession instrSession,
                                                  ViInt32 *rgstr)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "STAT:OPER?", 10, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;                                                           
	if (Scan (buffer, "%s>%d", rgstr) < 1)               
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	    
	return avd3371_status;   
}

/*===========================================================================*/
/* Function: Read Operation Status Condition Register                        */
/* Purpose:  This function reads the operation status condition resister.	 */
/*			 When this function is executed, this condition register is not  */
/*			 cleared.														 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Read_OperationCndRgstr (ViSession instrSession,
                                                 ViInt32 *rgstr)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "STAT:OPER:COND?", 15, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;                                                           
	if (Scan (buffer, "%s>%d", rgstr) < 1)               
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	    
	return avd3371_status;   
}

/*===========================================================================*/
/* Function: Set Questionable Status Enable Register                         */
/* Purpose:  This function sets the questionable status enable register.	 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Set_QuestionableEnblRgstr (ViSession instrSession,
                                         	        ViInt32 value)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avd3371_invalidViInt32Range (value, 0, 65535))
   		return VI_ERROR_PARAMETER2;
    
    Fmt (wrtBuffer, "%s<STAT:QUES:ENAB %d", value);
    
    if ((avd3371_status = viWrite (instrSession, wrtBuffer, NumFmtdBytes(), &retCount)) < 0)
        return avd3371_status;
    
    return avd3371_status;
}

/*===========================================================================*/
/* Function: Set Transition Filter of Questionable Status                    */
/* Purpose:  This function sets the transition filter of the questionable	 */
/*			 status.														 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Set_QuestionableTrnsFltr (ViSession instrSession,
                                          		   ViBoolean direct, ViInt32 value)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avd3371_invalidViBooleanRange (direct))
   		return VI_ERROR_PARAMETER2;
    if (avd3371_invalidViInt32Range (value, 0, 65535))
   		return VI_ERROR_PARAMETER3;
   		
   	Fmt (wrtBuffer, "%s<STAT:QUES:%s %d", directArr[direct], value);	
    
    if ((avd3371_status = viWrite (instrSession, wrtBuffer, NumFmtdBytes(), &retCount)) < 0)
        return avd3371_status;
    
    return avd3371_status;
}

/*===========================================================================*/
/* Function: Read Questionable Status Event Register                         */
/* Purpose:  This function reads questionable status event register. When    */
/*			 this function is executed, this register is cleared.			 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Read_QuestionableEvntRgstr (ViSession instrSession,
                                                     ViInt32 *rgstr)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "STAT:QUES?", 10, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;                                                           
	if (Scan (buffer, "%s>%d", rgstr) < 1)               
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	    
	return avd3371_status;   
}

/*===========================================================================*/
/* Function: Read Questionable Status Condition Register                     */
/* Purpose:  This function reads the questionable status condition resister. */
/*			 When this function is executed, this condition register is not  */
/*			 cleared.														 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Read_QuestionableCndRgstr (ViSession instrSession,
                                                    ViInt32 *rgstr)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "STAT:QUES:COND?", 15, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;                                                           
	if (Scan (buffer, "%s>%d", rgstr) < 1)               
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	    
	return avd3371_status;   
}

/*===========================================================================*/
/* Function: Set Settling Status Enable Register                             */
/* Purpose:  This function sets the settling status enable register.	 	 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Set_SettlingEnblRgstr (ViSession instrSession,
                                     	        ViInt32 value)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avd3371_invalidViInt32Range (value, 0, 65535))
   		return VI_ERROR_PARAMETER2;
    
    Fmt (wrtBuffer, "%s<STAT:OPER:SETT:ENAB %d", value);
    
    if ((avd3371_status = viWrite (instrSession, wrtBuffer, NumFmtdBytes(), &retCount)) < 0)
        return avd3371_status;
    
    return avd3371_status;
}

/*===========================================================================*/
/* Function: Set Transition Filter of Settling Status                        */
/* Purpose:  This function sets the transition filter of the settling		 */
/*			 status.														 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Set_SettlingTrnsFltr (ViSession instrSession,
                                     		   ViBoolean direct, ViInt32 value)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avd3371_invalidViBooleanRange (direct))
   		return VI_ERROR_PARAMETER2;
    if (avd3371_invalidViInt32Range (value, 0, 65535))
   		return VI_ERROR_PARAMETER3;
   		
   	Fmt (wrtBuffer, "%s<STAT:OPER:SETT:%s %d", directArr[direct], value);	
    
    if ((avd3371_status = viWrite (instrSession, wrtBuffer, NumFmtdBytes(), &retCount)) < 0)
        return avd3371_status;
    
    return avd3371_status;
}

/*===========================================================================*/
/* Function: Read Settling Status Event Register                             */
/* Purpose:  This function reads settling status event register. When this   */
/*			 function is executed, this register is cleared.				 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Read_SettlingEvntRgstr (ViSession instrSession,
                                                 ViInt32 *rgstr)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "STAT:OPER:SETT?", 15, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;                                                           
	if (Scan (buffer, "%s>%d", rgstr) < 1)               
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	    
	return avd3371_status;   
}

/*===========================================================================*/
/* Function: Read Settling Status Condition Register                         */
/* Purpose:  This function reads the settling status condition resister.	 */
/*			 When this function is executed, this condition register is not  */
/*			 cleared.														 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Read_SettlingCndRgstr (ViSession instrSession,
                                                ViInt32 *rgstr)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "STAT:OPER:SETT:COND?", 20, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;                                                           
	if (Scan (buffer, "%s>%d", rgstr) < 1)               
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	    
	return avd3371_status;   
}

/*===========================================================================*/
/* Function: Set Clock Loss Status Enable Register                           */
/* Purpose:  This function sets the clock loss status enable register.	 	 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Set_CLOSSEnblRgstr (ViSession instrSession,
                                             ViInt32 value)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avd3371_invalidViInt32Range (value, 0, 65535))
   		return VI_ERROR_PARAMETER2;
    
    Fmt (wrtBuffer, "%s<STAT:QUES:CLOS:ENAB %d", value);
    
    if ((avd3371_status = viWrite (instrSession, wrtBuffer, NumFmtdBytes(), &retCount)) < 0)
        return avd3371_status;
    
    return avd3371_status;
}

/*===========================================================================*/
/* Function: Set Transition Filter of Clock Loss Status                      */
/* Purpose:  This function sets the transition filter of the clock loss		 */
/*			 status.														 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Set_CLOSSTrnsFltr (ViSession instrSession,
                                            ViBoolean direct, ViInt32 value)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avd3371_invalidViBooleanRange (direct))
   		return VI_ERROR_PARAMETER2;
    if (avd3371_invalidViInt32Range (value, 0, 65535))
   		return VI_ERROR_PARAMETER3;
   		
   	Fmt (wrtBuffer, "%s<STAT:QUES:CLOS:%s %d", directArr[direct], value);	
    
    if ((avd3371_status = viWrite (instrSession, wrtBuffer, NumFmtdBytes(), &retCount)) < 0)
        return avd3371_status;
    
    return avd3371_status;
}

/*===========================================================================*/
/* Function: Read Clock Loss Status Event Register                           */
/* Purpose:  This function reads clock loss status event register. When this */
/*			 function is executed, this register is cleared.				 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Read_CLOSSEvntRgstr (ViSession instrSession,
                                              ViInt32 *rgstr)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "STAT:QUES:CLOS?", 15, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;                                                           
	if (Scan (buffer, "%s>%d", rgstr) < 1)               
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	    
	return avd3371_status;   
}

/*===========================================================================*/
/* Function: Read Clock Loss Status Condition Register                       */
/* Purpose:  This function reads the clock loss status condition resister.	 */
/*			 When this function is executed, this condition register is not  */
/*			 cleared.														 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Read_CLOSSCndRgstr (ViSession instrSession,
                                             ViInt32 *rgstr)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "STAT:QUES:CLOS:COND?", 20, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;                                                           
	if (Scan (buffer, "%s>%d", rgstr) < 1)               
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	    
	return avd3371_status;   
}

/*===========================================================================*/
/* Function: Read Device Status Event Register                               */
/* Purpose:  This function reads device status event register. When this	 */
/*			 function is executed, this register is cleared.				 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Read_DeviceEvntRgstr (ViSession instrSession,
                                               ViInt32 *rgstr)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "STAT:DEV?", 9, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;                                                           
	if (Scan (buffer, "%s>%d", rgstr) < 1)               
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	    
	return avd3371_status;   
}

/*===========================================================================*/
/* Function: Initialize Status Register		                                 */
/* Purpose:  This function initializes status register.						 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_Init_StatusRegister (ViSession instrSession)
{
	ViStatus avd3371_status = VI_SUCCESS;
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "STAT:PRES", 9, &retCount)) < 0)
        return avd3371_status;
    
    return avd3371_status;
}

/*===========================================================================*/
/* Function: On/Off Output					                                 */
/* Purpose:  This function switches simultaneously 4 outputs. 4 outputs are  */
/*			 data, xdata, clock and xclock.									 */
/*===========================================================================*/
ViStatus _VI_FUNC avd3371_OnOff_Outputs (ViSession instrSession, ViBoolean out)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avd3371_invalidViBooleanRange (out))
        return VI_ERROR_PARAMETER2;
        
    Fmt (wrtBuffer, "%s<OUTP %d;:OUTP3 %d;:OUTP2 %d;:OUTP4 %d", out, out, out, out);
    
    if ((avd3371_status = viWrite (instrSession, wrtBuffer, NumFmtdBytes(), &retCount)) < 0)
        return avd3371_status;
    
    return avd3371_status;    
}

/*===========================================================================*/
/* Function: Single Error Addition			                                 */
/* Purpose:  This function add 1 bit error. This function is effective, when */
/*			 [PPG]-[Error Addition]-[Add Errors] in [Setting] window is sets */
/*			 to [Single].													 */
/*===========================================================================*/
ViStatus _VI_FUNC avd3371_SingleErrorAddition (ViSession instrSession)
{
	ViStatus avd3371_status = VI_SUCCESS;
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "SOUR:EADD:SING", 14, &retCount)) < 0)
        return avd3371_status;
    
    return avd3371_status;
}

/*===========================================================================*/
/* Function: Re-Synchronization				                                 */
/* Purpose:  This function executes re-synchronization of the ED module's	 */
/*			 pattern.														 */
/*===========================================================================*/
ViStatus _VI_FUNC avd3371_ReSync (ViSession instrSession)
{
	ViStatus avd3371_status = VI_SUCCESS;
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "SENS:SYNC ONCE", 14, &retCount)) < 0)
        return avd3371_status;
    
    return avd3371_status;
}

/*===========================================================================*/
/* Function: Clear History					                                 */
/* Purpose:  This function clears the history information of the monitor 	 */
/*			 tool bar.														 */
/*===========================================================================*/
ViStatus _VI_FUNC avd3371_ClearHistory (ViSession instrSession)
{
	ViStatus avd3371_status = VI_SUCCESS;
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "SENS:MON:HIST:CLE", 17, &retCount)) < 0)
        return avd3371_status;
    
    return avd3371_status;
    
}

/*===========================================================================*/
/* Function: Auto Search Result				                                 */
/* Purpose:  This function returns the result of the auto search.			 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_AutoSearchResult (ViSession instrSession,
                                           ViInt16 *result)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "SENS:ASE:FAIL?", 14, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;                                                           
	if (Scan (buffer, "%s>%d[b2]", result) < 1)               
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	    
	return avd3371_status;   
}

/*===========================================================================*/
/* Function: Auto Search Data				                                 */
/* Purpose:  This function returns the data of auto search.					 */
/*===========================================================================*/
ViStatus _VI_FUNC avd3371_AutoSearchData (ViSession instrSession,
                                          ViChar inputPol[], ViReal64 *thresVol,
                                          ViInt16 *delay)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "INP:POL?", 8, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (Scan (buffer, "%s>%s", inputPol) < 1)               
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

	if ((avd3371_status = viWrite (instrSession, "INP:TVOL?", 9, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (Scan (buffer, "%s>%f", thresVol) < 1)               
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	    
	if ((avd3371_status = viWrite (instrSession, "INP2:DEL?", 9, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (Scan (buffer, "%s>%d[b2]", delay) < 1)               
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	    
return avd3371_status;   
}

/*===========================================================================*/
/* Function: Bit Count Fetch Data			                                 */
/* Purpose:  This function returns the measurement results of the bit count  */
/*			 data.															 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_BitCountFetchData (ViSession instrSession,
                                            ViReal64 *ttlCntEXP, ViInt32 *ttlCntINT,
                                            ViReal64 *omtCntEXP, ViInt32 *omtCntINT,
                                            ViReal64 *insCntEXP, ViInt32 *insCntINT)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "FETC:BME:BCO?", 13, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (dispBitCnt==DISP_DBL)
	{
		if (Scan (buffer, "%s>%f[e]", ttlCntEXP) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*ttlCntINT = 0;   
	}
	else
	{
		if (Scan (buffer, "%s>%d", ttlCntINT) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*ttlCntEXP = 0.0;  
	}
    
    if ((avd3371_status = viWrite (instrSession, "FETC:BME:BCO:OMIT?", 18, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (dispBitCnt==DISP_DBL)
	{
		if (Scan (buffer, "%s>%f[e]", omtCntEXP) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*omtCntINT = 0;   
	}
	else
	{
		if (Scan (buffer, "%s>%d", omtCntINT) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*omtCntEXP = 0.0;  
	}

    if ((avd3371_status = viWrite (instrSession, "FETC:BME:BCO:INS?", 17, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (dispBitCnt==DISP_DBL)
	{
		if (Scan (buffer, "%s>%f[e]", insCntEXP) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*insCntINT = 0;   
	}
	else
	{
		if (Scan (buffer, "%s>%d", insCntINT) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*insCntEXP = 0.0;  
	}

	return avd3371_status;
}

/*===========================================================================*/
/* Function: Bit Count Current Data			                                 */
/* Purpose:  This function returns the current data of the bit count. 		 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_BitCountCurrentData (ViSession instrSession,
                                              ViReal64 *ttlCntEXP, ViInt32 *ttlCntINT,
                                              ViReal64 *omtCntEXP, ViInt32 *omtCntINT,
                                              ViReal64 *insCntEXP, ViInt32 *insCntINT)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "CFET:BME:BCO?", 13, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (dispBitCnt==DISP_DBL)
	{
		if (Scan (buffer, "%s>%f[e]", ttlCntEXP) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*ttlCntINT = 0;   
	}
	else
	{
		if (Scan (buffer, "%s>%d", ttlCntINT) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*ttlCntEXP = 0.0;  
	}
    
    if ((avd3371_status = viWrite (instrSession, "CFET:BME:BCO:OMIT?", 18, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (dispBitCnt==DISP_DBL)
	{
		if (Scan (buffer, "%s>%f[e]", omtCntEXP) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*omtCntINT = 0;   
	}
	else
	{
		if (Scan (buffer, "%s>%d", omtCntINT) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*omtCntEXP = 0.0;  
	}

    if ((avd3371_status = viWrite (instrSession, "CFET:BME:BCO:INS?", 17, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (dispBitCnt==DISP_DBL)
	{
		if (Scan (buffer, "%s>%f[e]", insCntEXP) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*insCntINT = 0;   
	}
	else
	{
		if (Scan (buffer, "%s>%d", insCntINT) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*insCntEXP = 0.0;  
	}

	return avd3371_status;
}

/*===========================================================================*/
/* Function: Error Rate Fetch Data			                                 */
/* Purpose:  This function returns the measurement results of the error		 */
/*			 rate data.														 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_ErrorRateFetchData (ViSession instrSession,
                                             ViReal64 *ttlErr, ViReal64 *omtErr,
                                             ViReal64 *insErr)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "FETC:BME:ERAT?", 14, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (Scan (buffer, "%s>%f[e]", ttlErr) < 1)               
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    
    if ((avd3371_status = viWrite (instrSession, "FETC:BME:ERAT:OMIT?", 19, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (Scan (buffer, "%s>%f[e]", omtErr) < 1)               
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avd3371_status = viWrite (instrSession, "FETC:BME:ERAT:INS?", 18, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (Scan (buffer, "%s>%f[e]", insErr) < 1)               
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

	return avd3371_status;
}

/*===========================================================================*/
/* Function: Error Rate Current Data		                                 */
/* Purpose:  This function returns the current data of the error rate.		 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_ErrorRateCurrData (ViSession instrSession,
                                            ViReal64 *ttlErr, ViReal64 *omtErr,
                                            ViReal64 *insErr)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "CFET:BME:ERAT?", 14, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (Scan (buffer, "%s>%f[e]", ttlErr) < 1)               
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    
    if ((avd3371_status = viWrite (instrSession, "CFET:BME:ERAT:OMIT?", 19, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (Scan (buffer, "%s>%f[e]", omtErr) < 1)               
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avd3371_status = viWrite (instrSession, "CFET:BME:ERAT:INS?", 18, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (Scan (buffer, "%s>%f[e]", insErr) < 1)               
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

	return avd3371_status;
}

/*===========================================================================*/
/* Function: Error Count Fetch Data			                                 */
/* Purpose:  This function returns the measurement results of the error		 */
/*			 count data.													 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_ErrorCountFetchData (ViSession instrSession,
                                              ViReal64 *ttlCntEXP, ViInt32 *ttlCntINT,
                                              ViReal64 *omtCntEXP, ViInt32 *omtCntINT,
                                              ViReal64 *insCntEXP, ViInt32 *insCntINT)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "FETC:BME:ECO?", 13, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (dispErrCnt==DISP_DBL)
	{
		if (Scan (buffer, "%s>%f[e]", ttlCntEXP) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*ttlCntINT = 0;   
	}
	else
	{
		if (Scan (buffer, "%s>%d", ttlCntINT) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*ttlCntEXP = 0.0;  
	}
    
    if ((avd3371_status = viWrite (instrSession, "FETC:BME:ECO:OMIT?", 18, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (dispErrCnt==DISP_DBL)
	{
		if (Scan (buffer, "%s>%f[e]", omtCntEXP) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*omtCntINT = 0;   
	}
	else
	{
		if (Scan (buffer, "%s>%d", omtCntINT) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*omtCntEXP = 0.0;  
	}

    if ((avd3371_status = viWrite (instrSession, "FETC:BME:ECO:INS?", 17, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (dispErrCnt==DISP_DBL)
	{
		if (Scan (buffer, "%s>%f[e]", insCntEXP) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*insCntINT = 0;   
	}
	else
	{
		if (Scan (buffer, "%s>%d", insCntINT) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*insCntEXP = 0.0;  
	}

	return avd3371_status;
}

/*===========================================================================*/
/* Function: Error Count Current Data			                             */
/* Purpose:  This function returns the current data of the error count.		 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_ErrorCountCurrData (ViSession instrSession,
                                          	 ViReal64 *ttlCntEXP, ViInt32 *ttlCntINT,
                                             ViReal64 *omtCntEXP, ViInt32 *omtCntINT,
                                             ViReal64 *insCntEXP, ViInt32 *insCntINT)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "CFET:BME:ECO?", 13, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (dispErrCnt==DISP_DBL)
	{
		if (Scan (buffer, "%s>%f[e]", ttlCntEXP) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*ttlCntINT = 0;   
	}
	else
	{
		if (Scan (buffer, "%s>%d", ttlCntINT) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*ttlCntEXP = 0.0;  
	}
    
    if ((avd3371_status = viWrite (instrSession, "CFET:BME:ECO:OMIT?", 18, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (dispErrCnt==DISP_DBL)
	{
		if (Scan (buffer, "%s>%f[e]", omtCntEXP) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*omtCntINT = 0;   
	}
	else
	{
		if (Scan (buffer, "%s>%d", omtCntINT) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*omtCntEXP = 0.0;  
	}

    if ((avd3371_status = viWrite (instrSession, "CFET:BME:ECO:INS?", 17, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (dispErrCnt==DISP_DBL)
	{
		if (Scan (buffer, "%s>%f[e]", insCntEXP) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*insCntINT = 0;   
	}
	else
	{
		if (Scan (buffer, "%s>%d", insCntINT) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*insCntEXP = 0.0;  
	}

	return avd3371_status;
}

/*===========================================================================*/
/* Function: Immediate Error Rate Data			                             */
/* Purpose:  This function returns the immediate error rate data.			 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_ImmErrorRateData (ViSession instrSession,
                                           ViReal64 *ttlErr, ViReal64 *omtErr,
                                           ViReal64 *insErr)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "CFET:BME:IER?", 13, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (Scan (buffer, "%s>%f[e]", ttlErr) < 1)               
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    
    if ((avd3371_status = viWrite (instrSession, "CFET:BME:IER:OMIT?", 18, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (Scan (buffer, "%s>%f[e]", omtErr) < 1)               
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avd3371_status = viWrite (instrSession, "CFET:BME:IER:INS?", 17, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (Scan (buffer, "%s>%f[e]", insErr) < 1)               
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

	return avd3371_status;
}

/*===========================================================================*/
/* Function: Immediate Error Count Data			                             */
/* Purpose:  This function returns the immediate error count data.			 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_ImmErrorCountData (ViSession instrSession,
                                          	ViReal64 *ttlCntEXP, ViInt32 *ttlCntINT,
                                            ViReal64 *omtCntEXP, ViInt32 *omtCntINT,
                                            ViReal64 *insCntEXP, ViInt32 *insCntINT)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "CFET:BME:IEC?", 13, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (dispImmErrCnt==DISP_DBL)
	{
		if (Scan (buffer, "%s>%f[e]", ttlCntEXP) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*ttlCntINT = 0;   
	}
	else
	{
		if (Scan (buffer, "%s>%d", ttlCntINT) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*ttlCntEXP = 0.0;  
	}
    
    if ((avd3371_status = viWrite (instrSession, "CFET:BME:IEC:OMIT?", 18, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (dispImmErrCnt==DISP_DBL)
	{
		if (Scan (buffer, "%s>%f[e]", omtCntEXP) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*omtCntINT = 0;   
	}
	else
	{
		if (Scan (buffer, "%s>%d", omtCntINT) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*omtCntEXP = 0.0;  
	}

    if ((avd3371_status = viWrite (instrSession, "CFET:BME:IEC:INS?", 17, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (dispImmErrCnt==DISP_DBL)
	{
		if (Scan (buffer, "%s>%f[e]", insCntEXP) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*insCntINT = 0;   
	}
	else
	{
		if (Scan (buffer, "%s>%d", insCntINT) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*insCntEXP = 0.0;  
	}

	return avd3371_status;
}

/*===========================================================================*/
/* Function: Error Intervals Fetch Data		                                 */
/* Purpose:  This function returns the measurement results of the error		 */
/*			 intervals.														 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_ErrorIntervalFetchData (ViSession instrSession,
                                                 ViReal64 *ttlValPRC, ViInt32 *ttlValINT,
                                                 ViReal64 *omtValPRC, ViInt32 *omtValINT,
                                                 ViReal64 *insValPRC, ViInt32 *insValINT)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "FETC:BME:EINT?", 14, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (dispErrIntrvl==DISP_DBL)
	{
		if (Scan (buffer, "%s>%f[e]", ttlValPRC) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*ttlValINT = 0;   
	}
	else
	{
		if (Scan (buffer, "%s>%d", ttlValINT) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*ttlValPRC = 0.0;  
	}
    
    if ((avd3371_status = viWrite (instrSession, "FETC:BME:EINT:OMIT?", 19, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (dispErrIntrvl==DISP_DBL)
	{
		if (Scan (buffer, "%s>%f[e]", omtValPRC) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*omtValINT = 0;   
	}
	else
	{
		if (Scan (buffer, "%s>%d", omtValINT) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*omtValPRC = 0.0;  
	}

    if ((avd3371_status = viWrite (instrSession, "FETC:BME:EINT:INS?", 18, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (dispErrIntrvl==DISP_DBL)
	{
		if (Scan (buffer, "%s>%f[e]", insValPRC) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*insValINT = 0;   
	}
	else
	{
		if (Scan (buffer, "%s>%d", insValINT) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*insValPRC = 0.0;  
	}

	return avd3371_status;
}

/*===========================================================================*/
/* Function: Error Intervals Current Data	                                 */
/* Purpose:  This function returns the current data of the error intervals.	 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_ErrorIntervalCurrData (ViSession instrSession,
                                                ViReal64 *ttlValPRC, ViInt32 *ttlValINT,
                                                ViReal64 *omtValPRC, ViInt32 *omtValINT,
                                                ViReal64 *insValPRC, ViInt32 *insValINT)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "CFET:BME:EINT?", 14, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (dispErrIntrvl==DISP_DBL)
	{
		if (Scan (buffer, "%s>%f[e]", ttlValPRC) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*ttlValINT = 0;   
	}
	else
	{
		if (Scan (buffer, "%s>%d", ttlValINT) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*ttlValPRC = 0.0;  
	}
    
    if ((avd3371_status = viWrite (instrSession, "CFET:BME:EINT:OMIT?", 19, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (dispErrIntrvl==DISP_DBL)
	{
		if (Scan (buffer, "%s>%f[e]", omtValPRC) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*omtValINT = 0;   
	}
	else
	{
		if (Scan (buffer, "%s>%d", omtValINT) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*omtValPRC = 0.0;  
	}

    if ((avd3371_status = viWrite (instrSession, "CFET:BME:EINT:INS?", 18, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (dispErrIntrvl==DISP_DBL)
	{
		if (Scan (buffer, "%s>%f[e]", insValPRC) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*insValINT = 0;   
	}
	else
	{
		if (Scan (buffer, "%s>%d", insValINT) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*insValPRC = 0.0;  
	}

	return avd3371_status;
}

/*===========================================================================*/
/* Function: Error Free Intervals Fetch Data                                 */
/* Purpose:  This function returns the measurement results of the error		 */
/*			 free intervals.												 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_ErrFreeIntrvlFetchData (ViSession instrSession,
                                                 ViReal64 *ttlValPRC, ViInt32 *ttlValINT,
                                                 ViReal64 *omtValPRC, ViInt32 *omtValINT,
                                                 ViReal64 *insValPRC, ViInt32 *insValINT)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "FETC:BME:EFIN?", 14, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (dispErrFrIntrvl==DISP_DBL)
	{
		if (Scan (buffer, "%s>%f[e]", ttlValPRC) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*ttlValINT = 0;   
	}
	else
	{
		if (Scan (buffer, "%s>%d", ttlValINT) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*ttlValPRC = 0.0;  
	}
    
    if ((avd3371_status = viWrite (instrSession, "FETC:BME:EFIN:OMIT?", 19, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (dispErrFrIntrvl==DISP_DBL)
	{
		if (Scan (buffer, "%s>%f[e]", omtValPRC) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*omtValINT = 0;   
	}
	else
	{
		if (Scan (buffer, "%s>%d", omtValINT) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*omtValPRC = 0.0;  
	}

    if ((avd3371_status = viWrite (instrSession, "FETC:BME:EFIN:INS?", 18, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (dispErrFrIntrvl==DISP_DBL)
	{
		if (Scan (buffer, "%s>%f[e]", insValPRC) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*insValINT = 0;   
	}
	else
	{
		if (Scan (buffer, "%s>%d", insValINT) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*insValPRC = 0.0;  
	}

	return avd3371_status;
}

/*===========================================================================*/
/* Function: Error Free Intervals Current Data                               */
/* Purpose:  This function returns the current data of the error free		 */
/*			 intervals.														 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_ErrFreeIntrvlCurrData (ViSession instrSession,
                                                ViReal64 *ttlValPRC, ViInt32 *ttlValINT,
                                                ViReal64 *omtValPRC, ViInt32 *omtValINT,
                                                ViReal64 *insValPRC, ViInt32 *insValINT)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "CFET:BME:EFIN?", 14, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (dispErrFrIntrvl==DISP_DBL)
	{
		if (Scan (buffer, "%s>%f[e]", ttlValPRC) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*ttlValINT = 0;   
	}
	else
	{
		if (Scan (buffer, "%s>%d", ttlValINT) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*ttlValPRC = 0.0;  
	}
    
    if ((avd3371_status = viWrite (instrSession, "CFET:BME:EFIN:OMIT?", 19, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (dispErrFrIntrvl==DISP_DBL)
	{
		if (Scan (buffer, "%s>%f[e]", omtValPRC) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*omtValINT = 0;   
	}
	else
	{
		if (Scan (buffer, "%s>%d", omtValINT) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*omtValPRC = 0.0;  
	}

    if ((avd3371_status = viWrite (instrSession, "CFET:BME:EFIN:INS?", 18, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (dispErrFrIntrvl==DISP_DBL)
	{
		if (Scan (buffer, "%s>%f[e]", insValPRC) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*insValINT = 0;   
	}
	else
	{
		if (Scan (buffer, "%s>%d", insValINT) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*insValPRC = 0.0;  
	}

	return avd3371_status;
}

/*===========================================================================*/
/* Function: Frequency Fetch Data			                                 */
/* Purpose:  This function returns the measured	frequency.					 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_FreqFetchData (ViSession instrSession, ViReal64 *freq)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "FETC:BME:FREQ?", 14, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (Scan (buffer, "%s>%f", freq) < 1)               
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

	return avd3371_status;
}

/*===========================================================================*/
/* Function: Frequency Current Data			                                 */
/* Purpose:  This function returns the current frequency.					 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_FreqCurrData (ViSession instrSession, ViReal64 *freq)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "CFET:BME:FREQ?", 14, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (Scan (buffer, "%s>%f", freq) < 1)               
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

	return avd3371_status;
}

/*===========================================================================*/
/* Function: Clock Loss Intervals Fetch Data	                             */
/* Purpose:  This function returns the measured count number of clock loss   */
/*			 intervals.														 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_ClockLossFetchData (ViSession instrSession, 
											 ViInt32 *number)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "FETC:BME:CLIN?", 14, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (Scan (buffer, "%s>%d", number) < 1)               
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

	return avd3371_status;
}

/*===========================================================================*/
/* Function: Clock Loss Intervals Current Data	                             */
/* Purpose:  This function returns the count number of clock loss intervals  */
/*			 currently														 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_ClockLossCurrData (ViSession instrSession,
                                            ViInt32 *number)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "CFET:BME:CLIN?", 14, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (Scan (buffer, "%s>%d", number) < 1)               
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

	return avd3371_status;
}

/*===========================================================================*/
/* Function: Sync Loss Intervals Fetch Data		                             */
/* Purpose:  This function returns the measured count number of sync loss	 */
/*			 intervals.														 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_SyncLossFetchData (ViSession instrSession,
                                            ViInt32 *number)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "FETC:BME:SLIN?", 14, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (Scan (buffer, "%s>%d", number) < 1)               
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

	return avd3371_status;
}

/*===========================================================================*/
/* Function: Sync Loss Intervals Current Data	                             */
/* Purpose:  This function returns the count number of sync loss intervals	 */
/*			 currently														 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_SyncLossCurrData (ViSession instrSession,
                                           ViInt32 *number)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "CFET:BME:SLIN?", 14, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (Scan (buffer, "%s>%d", number) < 1)               
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

	return avd3371_status;
}

/*===========================================================================*/
/* Function: Threshold EI Fetch Data			                             */
/* Purpose:  This function returns the measured rate or count number of 	 */
/*			 error intervals versus selected threshold condition.			 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_ThresholdEIFetchData (ViSession instrSession, 
											   ViInt16 selCond,
                                               ViReal64 *valPRC, ViInt32 *valINT)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avd3371_invalidViInt16Range (selCond, 0, 7))
        return VI_ERROR_PARAMETER2;
        
    Fmt (buffer, "%s<FETC:BME:TEIN:%s?", thresCndArr[selCond]);   
    if ((avd3371_status = viWrite (instrSession, buffer, NumFmtdBytes(), &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (dispThresEI==DISP_DBL)
	{
		if (Scan (buffer, "%s>%f[e]", valPRC) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*valINT = 0;   
	}
	else
	{
		if (Scan (buffer, "%s>%d", valINT) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*valPRC = 0.0;  
	}

	return avd3371_status;
}

/*===========================================================================*/
/* Function: Threshold EI Current Data			                             */
/* Purpose:  This function returns the current rate or count number of	 	 */
/*			 error intervals versus selected threshold condition.			 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_ThresholdEICurrData (ViSession instrSession,
											  ViInt16 selCond,
                                              ViReal64 *valPRC, ViInt32 *valINT)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avd3371_invalidViInt16Range (selCond, 0, 7))
        return VI_ERROR_PARAMETER2;
        
    Fmt (buffer, "%s<CFET:BME:TEIN:%s?", thresCndArr[selCond]);   
    if ((avd3371_status = viWrite (instrSession, buffer, NumFmtdBytes(), &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (dispThresEI==DISP_DBL)
	{
		if (Scan (buffer, "%s>%f[e]", valPRC) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*valINT = 0;   
	}
	else
	{
		if (Scan (buffer, "%s>%d", valINT) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*valPRC = 0.0;  
	}

	return avd3371_status;
}

/*===========================================================================*/
/* Function: Threshold EFI Fetch Data			                             */
/* Purpose:  This function returns the measured rate or count number of 	 */
/*			 error free intervals versus selected threshold condition.		 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_ThresholdEFIFetchData (ViSession instrSession,
												ViInt16 selCond,
                                            	ViReal64 *valPRC, ViInt32 *valINT)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avd3371_invalidViInt16Range (selCond, 0, 7))
        return VI_ERROR_PARAMETER2;
        
    Fmt (buffer, "%s<FETC:BME:TEF:%s?", thresCndArr[selCond]);   
    if ((avd3371_status = viWrite (instrSession, buffer, NumFmtdBytes(), &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (dispThresEFI==DISP_DBL)
	{
		if (Scan (buffer, "%s>%f[e]", valPRC) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*valINT = 0;   
	}
	else
	{
		if (Scan (buffer, "%s>%d", valINT) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*valPRC = 0.0;  
	}

	return avd3371_status;
}

/*===========================================================================*/
/* Function: Threshold EFI Current Data			                             */
/* Purpose:  This function returns the current rate or count number of	 	 */
/*			 error free intervals versus selected threshold condition.		 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_ThresholdEFICurrData (ViSession instrSession,
											   ViInt16 selCond,
                                               ViReal64 *valPRC, ViInt32 *valINT)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avd3371_invalidViInt16Range (selCond, 0, 7))
        return VI_ERROR_PARAMETER2;
        
    Fmt (buffer, "%s<CFET:BME:TEF:%s?", thresCndArr[selCond]);   
    if ((avd3371_status = viWrite (instrSession, buffer, NumFmtdBytes(), &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (dispThresEFI==DISP_DBL)
	{
		if (Scan (buffer, "%s>%f[e]", valPRC) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*valINT = 0;   
	}
	else
	{
		if (Scan (buffer, "%s>%d", valINT) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*valPRC = 0.0;  
	}

	return avd3371_status;
}

/*===========================================================================*/
/* Function: Error Performance Fetch Data		                             */
/* Purpose:  This function returns the measurement error performance.		 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_ErrorPrfrmFetchData (ViSession instrSession,
                                              ViReal64 *errPRC, ViInt32 *errINT,
                                              ViReal64 *errFreePRC, ViInt32 *errFreeINT,
                                              ViReal64 *svrErrPRC, ViInt32 *svrErrINT,
                                              ViReal64 *unAvailPRC, ViInt32 *unAvailINT,
                                              ViReal64 *degradePRC, ViInt32 *degradeINT)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "FETC:BME:EPER:ESEC?", 19, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (dispErrPrfrm==DISP_DBL)
	{
		if (Scan (buffer, "%s>%f[e]", errPRC) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*errINT = 0;   
	}
	else
	{
		if (Scan (buffer, "%s>%d", errINT) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*errPRC = 0.0;  
	}
    
    if ((avd3371_status = viWrite (instrSession, "FETC:BME:EPER:EFS?", 18, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (dispErrPrfrm==DISP_DBL)
	{
		if (Scan (buffer, "%s>%f[e]", errFreePRC) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*errFreeINT = 0;   
	}
	else
	{
		if (Scan (buffer, "%s>%d", errFreeINT) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*errFreePRC = 0.0;  
	}
    
    if ((avd3371_status = viWrite (instrSession, "FETC:BME:EPER:SES?", 18, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (dispErrPrfrm==DISP_DBL)
	{
		if (Scan (buffer, "%s>%f[e]", svrErrPRC) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*svrErrINT = 0;   
	}
	else
	{
		if (Scan (buffer, "%s>%d", svrErrINT) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*svrErrPRC = 0.0;  
	}
    
    if ((avd3371_status = viWrite (instrSession, "FETC:BME:EPER:USEC?", 19, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (dispErrPrfrm==DISP_DBL)
	{
		if (Scan (buffer, "%s>%f[e]", unAvailPRC) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*unAvailINT = 0;   
	}
	else
	{
		if (Scan (buffer, "%s>%d", unAvailINT) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*unAvailPRC = 0.0;  
	}
    
    if ((avd3371_status = viWrite (instrSession, "FETC:BME:EPER:DMIN?", 19, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (dispErrPrfrm==DISP_DBL)
	{
		if (Scan (buffer, "%s>%f[e]", degradePRC) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*degradeINT = 0;   
	}
	else
	{
		if (Scan (buffer, "%s>%d", degradeINT) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*degradePRC = 0.0;  
	}
    
    return avd3371_status;
}

/*===========================================================================*/
/* Function: Error Performance Current Data		                             */
/* Purpose:  This function returns the current error performance.		 	 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_ErrorPrfrmCurrData (ViSession instrSession,
                                             ViReal64 *errPRC, ViInt32 *errINT,
                                             ViReal64 *errFreePRC, ViInt32 *errFreeINT,
                                             ViReal64 *svrErrPRC, ViInt32 *svrErrINT,
                                             ViReal64 *unAvailPRC, ViInt32 *unAvailINT,
                                             ViReal64 *degradePRC, ViInt32 *degradeINT)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "CFET:BME:EPER:ESEC?", 19, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (dispErrPrfrm==DISP_DBL)
	{
		if (Scan (buffer, "%s>%f[e]", errPRC) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*errINT = 0;   
	}
	else
	{
		if (Scan (buffer, "%s>%d", errINT) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*errPRC = 0.0;  
	}
    
    if ((avd3371_status = viWrite (instrSession, "CFET:BME:EPER:EFS?", 18, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (dispErrPrfrm==DISP_DBL)
	{
		if (Scan (buffer, "%s>%f[e]", errFreePRC) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*errFreeINT = 0;   
	}
	else
	{
		if (Scan (buffer, "%s>%d", errFreeINT) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*errFreePRC = 0.0;  
	}
    
    if ((avd3371_status = viWrite (instrSession, "CFET:BME:EPER:SES?", 18, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (dispErrPrfrm==DISP_DBL)
	{
		if (Scan (buffer, "%s>%f[e]", svrErrPRC) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*svrErrINT = 0;   
	}
	else
	{
		if (Scan (buffer, "%s>%d", svrErrINT) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*svrErrPRC = 0.0;  
	}
    
    if ((avd3371_status = viWrite (instrSession, "CFET:BME:EPER:USEC?", 19, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (dispErrPrfrm==DISP_DBL)
	{
		if (Scan (buffer, "%s>%f[e]", unAvailPRC) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*unAvailINT = 0;   
	}
	else
	{
		if (Scan (buffer, "%s>%d", unAvailINT) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*unAvailPRC = 0.0;  
	}
    
    if ((avd3371_status = viWrite (instrSession, "CFET:BME:EPER:DMIN?", 19, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (dispErrPrfrm==DISP_DBL)
	{
		if (Scan (buffer, "%s>%f[e]", degradePRC) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*degradeINT = 0;   
	}
	else
	{
		if (Scan (buffer, "%s>%d", degradeINT) < 1)               
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		*degradePRC = 0.0;  
	}
    
    return avd3371_status;
}

/*===========================================================================*/
/* Function: Measutement Elapsed Time			                             */
/* Purpose:  This function returns the elapsed time of the measurement.		 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_MeasPassageTime (ViSession instrSession,
                                          ViInt16 *day, ViInt16 *hour,
                                          ViInt16 *minute, ViInt16 *second)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "FETC:MTIM:ETIM?", 15, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (Scan (buffer, "%s>%d[b2],%d[b2],%d[b2],%d[b2]", day, hour, minute, second) < 4)               
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avd3371_status;
}

/*===========================================================================*/
/* Function: Measutement Remained Time			                             */
/* Purpose:  This function returns the remained time of the measurement.	 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_MeasRemainedTime (ViSession instrSession,
                                           ViInt16 *day, ViInt16 *hour,
                                           ViInt16 *minute, ViInt16 *second)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "FETC:MTIM:TIM?", 14, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (Scan (buffer, "%s>%d[b2],%d[b2],%d[b2],%d[b2]", day, hour, minute, second) < 4)               
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avd3371_status;
}

/*===========================================================================*/
/* Function: Measutement Start Time				                             */
/* Purpose:  This function returns the start time of the measurement.		 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_MeasStartTime (ViSession instrSession,
                                        ViInt16 *year, ViInt16 *month,
                                        ViInt16 *day, ViInt16 *hour,
                                        ViInt16 *minute, ViInt16 *second)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "FETC:MTIM:STIM?", 15, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (Scan (buffer, "%s>%d[b2],%d[b2],%d[b2],%d[b2],%d[b2],%d[b2]", year, month, day, hour, minute, second) < 6)               
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avd3371_status;
}

/*===========================================================================*/
/* Function: Present Time						                             */
/* Purpose:  This function returns the present time.						 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_PresentTime (ViSession instrSession, 
									  ViInt16 *year, ViInt16 *month, 
                                      ViInt16 *day, ViInt16 *hour, 
                                      ViInt16 *minute, ViInt16 *second)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "FETC:MTIM:RTIM?", 15, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (Scan (buffer, "%s>%d[b2],%d[b2],%d[b2],%d[b2],%d[b2],%d[b2]", year, month, day, hour, minute, second) < 6)               
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avd3371_status;
}

/*===========================================================================*/
/* Function: Monitor Status Real Data			                             */
/* Purpose:  This function returns the occurrence of the data error, clock   */
/*			 loss and sync loss in real time.								 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_MntrSttRealData (ViSession instrSession,
                                          ViBoolean *dataError,
                                          ViBoolean *clockLoss, ViBoolean *syncLoss)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "FETC:MON:RTIM:DERR?", 19, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (Scan (buffer, "%s>%d[b2]", dataError) < 1)               
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avd3371_status = viWrite (instrSession, "FETC:MON:RTIM:CLOS?", 19, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (Scan (buffer, "%s>%d[b2]", clockLoss) < 1)               
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avd3371_status = viWrite (instrSession, "FETC:MON:RTIM:SLOS?", 19, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (Scan (buffer, "%s>%d[b2]", syncLoss) < 1)               
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avd3371_status;
}

/*===========================================================================*/
/* Function: Monitor Status history Data		                             */
/* Purpose:  This function returns the occurrence of the data error, clock   */
/*			 loss and sync loss in the measurement.							 */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_MntrSttHistData (ViSession instrSession,
                                          ViBoolean *dataError,
                                          ViBoolean *clockLoss, ViBoolean *syncLoss)
{
	ViStatus avd3371_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if ((avd3371_status = viWrite (instrSession, "FETC:MON:HIST:DERR?", 19, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (Scan (buffer, "%s>%d[b2]", dataError) < 1)               
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avd3371_status = viWrite (instrSession, "FETC:MON:HIST:CLOS?", 19, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (Scan (buffer, "%s>%d[b2]", clockLoss) < 1)               
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avd3371_status = viWrite (instrSession, "FETC:MON:HIST:SLOS?", 19, &retCount)) < 0)                
	    return avd3371_status;                                                           
	if ((avd3371_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avd3371_status;
	if (Scan (buffer, "%s>%d[b2]", syncLoss) < 1)               
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avd3371_status;
}

/*===========================================================================*/
/* Function: Reset                                                           */
/* Purpose:  This function resets the instrument to the state when power is  */
/*			 set to on.														 */
/*===========================================================================*/
ViStatus _VI_FUNC avd3371_Reset (ViSession instrSession)
{
    ViStatus avd3371_status = VI_SUCCESS;
	ViUInt32 retCount = 0; 

    /*  Initialize the instrument to the state when power is set to on.  */
    if ((avd3371_status = viWrite (instrSession, "*RST", 4, &retCount)) < 0)
        return avd3371_status;

    return avd3371_status;
}

/*===========================================================================*/
/* Function: Error Message                                                   */
/* Purpose:  This function translates the error return value from the        */
/*           instrument driver into a user-readable string.                  */
/*===========================================================================*/
ViStatus _VI_FUNC avd3371_ErrorMessage (ViSession instrSession,
                                         ViStatus statusCode, ViChar errMessage[])
{
    ViStatus avd3371_status = VI_SUCCESS;
    ViInt16 i;
    static avd3371_tStringValPair statusDescArray[] = {
        {VI_WARN_NSUP_ID_QUERY,     "WARNING: ID Query not supported"},
        {VI_WARN_NSUP_RESET,        "WARNING: Reset not supported"},
        {VI_WARN_NSUP_SELF_TEST,    "WARNING: Self-test not supported"},
        {VI_WARN_NSUP_ERROR_QUERY,  "WARNING: Error Query not supported"},     
        {VI_WARN_NSUP_REV_QUERY,    "WARNING: Revision Query not supported"},
        {VI_ERROR_PARAMETER1,   "ERROR: Parameter 1 out of range"},
        {VI_ERROR_PARAMETER2,   "ERROR: Parameter 2 out of range"},
        {VI_ERROR_PARAMETER3,   "ERROR: Parameter 3 out of range"},
        {VI_ERROR_PARAMETER4,   "ERROR: Parameter 4 out of range"},
        {VI_ERROR_PARAMETER5,   "ERROR: Parameter 5 out of range"},
        {VI_ERROR_PARAMETER6,   "ERROR: Parameter 6 out of range"},
        {VI_ERROR_PARAMETER7,   "ERROR: Parameter 7 out of range"},
        {VI_ERROR_PARAMETER8,   "ERROR: Parameter 8 out of range"},
        {VI_ERROR_FAIL_ID_QUERY,"ERROR: Identification query failed"},
        {VI_ERROR_INV_RESPONSE, "ERROR: Interpreting instrument response"},
        {VI_ERROR_INSTR_FILE_OPEN,    "ERROR: Opening the specified file"},
        {VI_ERROR_INSTR_FILE_WRITE,   "ERROR: Writing to the specified file"},
        {VI_ERROR_INSTR_INTERPRETING_RESPONSE, "ERROR: Interpreting the instrument's response"},
        {VI_ERROR_INSTR_PARAMETER9 ,  "ERROR: Parameter 9 out of range"},
        {VI_ERROR_INSTR_PARAMETER10,  "ERROR: Parameter 10 out of range"},
        {VI_ERROR_INSTR_PARAMETER11,  "ERROR: Parameter 11 out of range"},
        {VI_ERROR_INSTR_PARAMETER12,  "ERROR: Parameter 12 out of range"},
        {VI_ERROR_INSTR_PARAMETER13,  "ERROR: Parameter 13 out of range"},
        {VI_ERROR_INSTR_PARAMETER14,  "ERROR: Parameter 14 out of range"},
        {VI_ERROR_INSTR_PARAMETER15,  "ERROR: Parameter 15 out of range"},
        {VI_ERROR_INSTR_PARAMETER16,  "ERROR: Parameter 16 out of range"},
        
        {VI_NULL, VI_NULL}
    };

    avd3371_status = viStatusDesc (instrSession, statusCode, errMessage);
    if (avd3371_status == VI_WARN_UNKNOWN_STATUS) {
        for (i = 0;statusDescArray[i].stringName;i++) {
            if (statusDescArray[i].stringVal == statusCode) {
                Fmt (errMessage, "%s<%s", statusDescArray[i].stringName);
                return (VI_SUCCESS);
            }
        }
        Fmt (errMessage, "%s<Unknown Error 0x%x[uw8p0]", statusCode);
        return (VI_WARN_UNKNOWN_STATUS);
    }
    
    avd3371_status = VI_SUCCESS;
    return avd3371_status;
}

/*===========================================================================*/
/* Function: Revision Query                                                  */
/* Purpose:  This function returns the driver and instrument revisions.      */
/*===========================================================================*/
ViStatus _VI_FUNC avd3371_Query_TypeRevision (ViSession instrSession,
                                               ViChar driverRev[], ViChar instrType[],
                                               ViChar instrCode[], ViChar instrRev[])
{
    ViStatus avd3371_status = VI_SUCCESS;
	ViChar rdBuffer[BUFFER_SIZE];
	ViUInt32 retCount = 0;    
	
    if ((avd3371_status = viWrite (instrSession, "*IDN?", 5, &retCount)) < 0)
        return avd3371_status;

    if ((avd3371_status = viRead (instrSession, rdBuffer, BUFFER_SIZE, &retCount)) < 0)
        return avd3371_status;

    if (Scan (rdBuffer, "%s>%s[dt44],%s[t44],%s[t44],%s", instrType, instrCode, instrRev) < 4)
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    Fmt (driverRev, "%s<%s", AVD3371_REVISION);
    
    return avd3371_status;   
}

/*===========================================================================*/
/* Function: Close                                                           */
/* Purpose:  This function closes the instrument.                            */
/*===========================================================================*/
ViStatus CVIFUNC avd3371_close (ViSession instrSession)
{
    ViSession rmSession;
    ViStatus avd3371_status = VI_SUCCESS;

    if ((avd3371_status = viGetAttribute (instrSession, VI_ATTR_RM_SESSION, &rmSession)) < 0)
        return avd3371_status;
            
    avd3371_status = viClose (instrSession);
    viClose (rmSession);

    return avd3371_status;
}

/*****************************************************************************/
/*= UTILITY ROUTINES (Non-Exportable Functions) =============================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Boolean Value Out Of Range - ViBoolean                          */
/* Purpose:  This function checks a Boolean to see if it is equal to VI_TRUE */
/*           or VI_FALSE. If the value is out of range, the return value is  */
/*           VI_TRUE, otherwise the return value is VI_FALSE.                */
/*===========================================================================*/
ViBoolean avd3371_invalidViBooleanRange (ViBoolean val)
{
    return ((val != VI_FALSE && val != VI_TRUE) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Short Signed Integer Value Out Of Range - ViInt16               */
/* Purpose:  This function checks a short signed integer value to see if it  */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean avd3371_invalidViInt16Range (ViInt16 val, ViInt16 min, ViInt16 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Signed Integer Value Out Of Range - ViInt32                */
/* Purpose:  This function checks a long signed integer value to see if it   */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean avd3371_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Unsigned Char Value Out Of Range - ViUInt8                      */
/* Purpose:  This function checks an unsigned char value to see if it        */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean avd3371_invalidViUInt8Range (ViUInt8 val, ViUInt8 min, ViUInt8 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Short Unsigned Integer Value Out Of Range - ViUInt16            */
/* Purpose:  This function checks a short unsigned integer value to see if it*/  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean avd3371_invalidViUInt16Range (ViUInt16 val, ViUInt16 min, ViUInt16 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Unsigned Integer Value Out Of Range - ViUInt32             */
/* Purpose:  This function checks a long unsigned integer value to see if it */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean avd3371_invalidViUInt32Range (ViUInt32 val, ViUInt32 min, ViUInt32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Real (Float) Value Out Of Range - ViReal32                      */
/* Purpose:  This function checks a real (float) value to see if it lies     */  
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViBoolean avd3371_invalidViReal32Range (ViReal32 val, ViReal32 min, ViReal32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Real (Double) Value Out Of Range - ViReal64                     */
/* Purpose:  This function checks a real (double) value to see if it lies    */  
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViBoolean avd3371_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}


/*****************************************************************************/
/*----------- INSERT INSTRUMENT-DEPENDENT UTILITY ROUTINES HERE -------------*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Initialize Clean Up                                             */
/* Purpose:  This function is used only by the avd3371_Initialize function. */
/*			 When an error is detected this function is called to close the  */
/*           open resource manager and instrument object sessions and to     */
/*           set the instrSession that is returned from avd3371_Initialize  */
/*			 to VI_NULL.                                                     */
/*===========================================================================*/
ViStatus avd3371_initCleanUp (ViSession openRMSession,
                ViPSession openInstrSession, ViStatus currentStatus)
{
    viClose (*openInstrSession);
    viClose (openRMSession);
    *openInstrSession = VI_NULL;
    
    return currentStatus;
}

/*****************************************************************************/
/*=== END INSTRUMENT DRIVER SOURCE CODE =====================================*/
/*****************************************************************************/
